# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import Response
from flask_api import status

# own libs
from hpOneView.exceptions import HPOneViewException
from oneview_redfish_toolkit.api.service_root import ServiceRoot
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import connection

service_root = Blueprint('service_root', __name__)


@service_root.route('/', methods=["GET"])
def get_service_root():
    """Gets ServiceRoot

        Recover OneView UUID from appliance and creates
        ServiceRoot redfish JSON
    """

    try:
        ov_ip = config.get_oneview_multiple_ips()[0]
        appliance_node_information = \
            connection.request_oneview(ov_ip,
                                       '/rest/appliance/nodeinfo/version')
        uuid = appliance_node_information['uuid']

        sr = ServiceRoot(uuid)
        json_str = sr.serialize()
        return Response(
            response=json_str,
            status=200,
            mimetype='application/json')
    except HPOneViewException as e:
        if e.oneview_response['errorCode'] == "RESOURCE_NOT_FOUND":
            logging.exception("Resource not found: {}".format(e))
            abort(status.HTTP_404_NOT_FOUND, "Appliance not found")
        else:
            logging.exception("OneView Exception: {}".format(e))
            abort(status.HTTP_500_INTERNAL_SERVER_ERROR)
    except Exception as e:
        logging.exception('ServiceRoot error: {}'.format(e))
        abort(status.HTTP_500_INTERNAL_SERVER_ERROR)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json
import logging
import logging.config
import ssl
import time

# 3rd party libs
from flask import request
from flask_api import status
from hpOneView.oneview_client import OneViewClient
from http.client import HTTPSConnection

# Modules own libs
from oneview_redfish_toolkit.api.errors import OneViewRedfishException
from oneview_redfish_toolkit import config


SERVICE_ROOT_ENDPOINTS = ["/redfish/v1",
                          "/redfish",
                          "/redfish/v1/odata",
                          "/redfish/v1/$metadata"]


def new_oneview_client(ip_oneview, username=None, password=None,
                       api_version=None):
    credentials = None

    if config.auth_mode_is_conf():
        credentials = config.get_credentials()

    if config.auth_mode_is_session():
        credentials = create_credentials(username, password)

    ov_config = create_oneview_config(ip=ip_oneview,
                                      api_version=api_version,
                                      credentials=credentials)
    try:
        oneview_client = OneViewClient(ov_config)
        return oneview_client
    except Exception:
        logging.exception("Failed to recover session based connection")
        raise


def is_service_root():
    if request.path.rstrip("/") in SERVICE_ROOT_ENDPOINTS:
        return True

    return False


def check_oneview_availability(oneview_ip):
    """Check OneView availability by doing a GET request to OneView"""
    attempts = 3
    retry_interval_sec = 3

    for attempt_counter in range(attempts):
        try:
            status_ov = request_oneview(oneview_ip, '/controller-state.json')

            if status_ov['state'] != 'OK':
                message = "OneView state is not OK at {}".format(
                    oneview_ip)
                raise OneViewRedfishException(message)

            return
        except Exception as e:
            logging.exception(
                'Attempt {} to check OneView availability. '
                'Error: {}'.format(attempt_counter + 1, e))

            if attempt_counter + 1 < attempts:
                time.sleep(retry_interval_sec)

    message = "After {} attempts OneView is unreachable at {}".format(
        attempts, oneview_ip)
    raise OneViewRedfishException(message)


def request_oneview(oneview_ip, rest_url):
    try:
        connection = HTTPSConnection(
            oneview_ip, context=ssl.SSLContext(ssl.PROTOCOL_TLSv1_2))

        connection.request(
            method='GET', url=rest_url,
            headers={'Content-Type': 'application/json',
                     'X-API-Version': config.get_api_version()}
            )

        response = connection.getresponse()

        if response.status != status.HTTP_200_OK:
            message = "OneView is unreachable at {}".format(
                oneview_ip)
            raise OneViewRedfishException(message)

        text_response = response.read().decode('UTF-8')
        json_response = json.loads(text_response)

        return json_response
    finally:
        connection.close()


def create_credentials(username, password):
    credentials = dict()

    try:
        login_domain, username = username.split("\\")
    except (AttributeError, ValueError):
        logging.debug("AuthLoginDomain not specified on session creation")
    else:
        credentials["authLoginDomain"] = login_domain

    credentials["userName"] = username
    credentials["password"] = password
    return credentials


def create_oneview_config(ip, api_version=None, credentials=None):
    """Creates a dict to pass as argument on creating a new OneViewClient"""
    ov_config = {}
    ov_config['ip'] = ip
    ov_config['api_version'] = config.get_api_version()

    if credentials:
        ov_config['credentials'] = credentials

    if api_version:
        ov_config['api_version'] = api_version

    return ov_config



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import Blueprint
from flask import g

from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.ethernet_interface_collection import \
    EthernetInterfaceCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder

ethernet_interface_collection = Blueprint(
    "ethernet_interface_collection", __name__)


@ethernet_interface_collection.route(
    ComputerSystem.BASE_URI + "/<server_profile_uuid>/EthernetInterfaces/",
    methods=["GET"])
def get_ethernet_interface_collection(server_profile_uuid):
    """Get the Redfish Ethernet Interfaces Collection.

    Return EthernetInterfaceCollection Redfish JSON.
    """
    profile = g.oneview_client.server_profiles.get_by_id(
        server_profile_uuid).data

    ethernet_collection = EthernetInterfaceCollection(profile)

    return ResponseBuilder.success(ethernet_collection)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection \
    import ResourceBlockCollection
import oneview_redfish_toolkit.api.status_mapping as status_mapping


class Processor(RedfishJsonValidator):
    """Creates a Processor Redfish dict

        Populates self.redfish with some hardcoded Processor
        values and data retrieved from Oneview.
    """

    SCHEMA_NAME = 'Processor'

    def __init__(self, server_hardware, processor_id):
        """Processor constructor

            Populates self.redfish with the some common contents
            and data from OneView server hardware.

            Args:
                server_hardware: server hardware dict from OneView
                processor_id: processor identifier
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = processor_id
        self.redfish["Name"] = "Processor " + processor_id
        self.redfish["Status"] = dict()
        state, health = status_mapping.\
            get_redfish_server_hardware_status_struct(server_hardware)
        self.redfish["Status"]["State"] = state
        self.redfish["Status"]["Health"] = health
        self.redfish["ProcessorType"] = "CPU"
        self.redfish["Model"] = server_hardware["processorType"]
        self.redfish["MaxSpeedMHz"] = server_hardware["processorSpeedMhz"]
        self.redfish["TotalCores"] = server_hardware["processorCoreCount"]

        self._fill_links(server_hardware)

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#Processor.Processor"
        self.redfish["@odata.id"] = \
            ResourceBlockCollection.BASE_URI + "/" \
            + server_hardware["uuid"] + "/Systems/1/Processors/" + processor_id

        self._validate()

    def _fill_links(self, server_hardware):
        self.redfish["Links"] = dict()

        self.redfish["Links"]["Chassis"] = dict()
        self.redfish["Links"]["Chassis"]["@odata.id"] = \
            "/redfish/v1/Chassis/" + server_hardware["uuid"]



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import abort
from flask import Blueprint
from flask import g
from flask_api import status

from hpOneView.exceptions import HPOneViewException

from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.ethernet_interface import EthernetInterface
from oneview_redfish_toolkit.api.resource_block import ResourceBlock
from oneview_redfish_toolkit.api.server_hardware_resource_block \
    import ServerHardwareResourceBlock
from oneview_redfish_toolkit.api.server_profile_template_resource_block \
    import ServerProfileTemplateResourceBlock
from oneview_redfish_toolkit.api.storage_resource_block \
    import StorageResourceBlock
from oneview_redfish_toolkit.blueprints.util.response_builder \
    import ResponseBuilder
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit.services.manager_service import \
    get_manager_uuid
from oneview_redfish_toolkit.services.zone_service import ZoneService
from oneview_redfish_toolkit.single_oneview_context import single_oneview

resource_block = Blueprint("resource_block", __name__)


@resource_block.route(ResourceBlock.BASE_URI + "/<uuid>", methods=["GET"])
@single_oneview
def get_resource_block(uuid):
    """Get the Redfish ResourceBlock for a given UUID.

        Return ResourceBlock redfish JSON for a given UUID.
        Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with ResourceBlock.
    """
    zone_service = ZoneService(g.oneview_client)
    resource = _get_oneview_resource(uuid)
    category = resource["category"]

    if category == "server-hardware":
        result_resource_block = _build_computer_system_resource_block(
            uuid, resource)

    elif category == "server-profile-templates":
        zone_ids = zone_service.get_zone_ids_by_templates([resource])
        if not zone_ids:
            abort(status.HTTP_404_NOT_FOUND,
                  "Zone not found to ResourceBlock {}".format(uuid))

        result_resource_block = \
            ServerProfileTemplateResourceBlock(uuid, resource, zone_ids)

    elif category == "drives":
        drive_uuid = resource["uri"].split("/")[-1]
        drive_index_trees_uri = \
            "/rest/index/trees/rest/drives/{}?parentDepth=3"
        drive_index_trees = g.oneview_client.connection.get(
            drive_index_trees_uri.format(drive_uuid))

        server_profile_templs = \
            g.oneview_client.server_profile_templates.get_all()

        zone_ids = zone_service.get_zone_ids_by_templates(
            server_profile_templs)

        result_resource_block = StorageResourceBlock(
            resource, drive_index_trees, zone_ids, None)

    elif category == "storage-volumes":
        volumeUri = resource["uri"]

        # filter STP's with volume attached
        server_profile_templs = \
            g.oneview_client.server_profile_templates.get_all()

        filter_spt = list(filter(lambda i: i["sanStorage"][
            "volumeAttachments"], server_profile_templs))

        spt_with_volume = [spt for spt in filter_spt for volume in
                           spt["sanStorage"]["volumeAttachments"]
                           if volume.get("volumeUri") == volumeUri]

        zone_ids = []
        if spt_with_volume:
            zone_ids = zone_service.get_zone_ids_by_templates(spt_with_volume)

        # filter SP's with volume attached
        volume_attachments = g.oneview_client. \
            storage_volume_attachments.get_all(
                filter="storageVolumeUri='{}'".format(volumeUri))
        server_profiles = [i.get('ownerUri') for i in volume_attachments]

        result_resource_block = StorageResourceBlock(
            resource, None, zone_ids, server_profiles)

    else:
        abort(status.HTTP_404_NOT_FOUND, 'Resource block not found')

    return ResponseBuilder.success(
        result_resource_block,
        {"ETag": "W/" + resource["eTag"]})


@resource_block.route(
    ResourceBlock.BASE_URI + "/<uuid>/Systems/1", methods=["GET"])
@single_oneview
def get_resource_block_computer_system(uuid):
    """Get Computer System of a Resource Block

        Return ResourceBlock Computer System redfish JSON for a given
        UUID. Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with ResourceBlock Computer System.
    """

    server_hardware = g.oneview_client.server_hardware.get_by_id(uuid).data
    manager_uuid = get_manager_uuid(uuid)

    computer_system = ComputerSystem.build_physical_system(
        server_hardware, manager_uuid)

    return ResponseBuilder.success(
        computer_system,
        {"ETag": "W/" + server_hardware["eTag"]})


@resource_block.route(
    ResourceBlock.BASE_URI + "/<uuid>/EthernetInterfaces/<id>",
    methods=["GET"])
@single_oneview
def get_resource_block_ethernet_interface(uuid, id):
    """Get the Redfish ResourceBlock of type Network for the given UUID and ID.

        Return ResourceBlock redfish JSON for a given UUID and ID.
        Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with ResourceBlock.
    """

    server_profile_template = \
        g.oneview_client.server_profile_templates.get_by_id(uuid).data

    conn_settings = server_profile_template["connectionSettings"]
    connection = None

    for conn in conn_settings["connections"]:
        if str(conn["id"]) == id:
            connection = conn
            break

    if not connection:
        abort(status.HTTP_404_NOT_FOUND, "Ethernet interface not found")

    network = g.oneview_client.index_resources.get(connection["networkUri"])

    ethernet_interface = EthernetInterface.build_resource_block(
        server_profile_template, connection, network)

    return ResponseBuilder.success(
        ethernet_interface,
        {"ETag": "W/" + server_profile_template["eTag"]})


def _get_oneview_resource(uuid):
    drives_param = "/rest/drives/" + uuid
    cached_categ = category_resource.get_category_by_resource_id(uuid) or \
        category_resource.get_category_by_resource_id(drives_param)
    if cached_categ:
        resource_uuid = uuid

        if cached_categ.resource == 'index_resources':
            resource_uuid = drives_param

        resource = getattr(g.oneview_client, cached_categ.resource)
        function = getattr(resource, cached_categ.function)
        result = function(resource_uuid)
        if isinstance(result, dict):
            return result
        else:
            return result.data

    categories = [
        {"func": g.oneview_client.server_hardware.get_by_id, "param": uuid},
        {"func": g.oneview_client.server_profile_templates.get_by_id, "param": uuid},
        {"func": g.oneview_client.index_resources.get, "param": drives_param},
        {"func": g.oneview_client.volumes.get_by_id, "param": uuid}
    ]

    for category in categories:
        try:

            resource = category["func"](category["param"])
            if isinstance(resource, dict):
                return resource

            return resource.data
        except HPOneViewException as e:

            if e.oneview_response["errorCode"] == 'RESOURCE_NOT_FOUND':
                pass
            else:

                raise  # Raise any unexpected errors

    abort(status.HTTP_404_NOT_FOUND,
          "Could not find resource block with id " + uuid)


def _build_computer_system_resource_block(uuid, server_hardware):
    eg_uri = server_hardware["serverGroupUri"]
    sht_uri = server_hardware["serverHardwareTypeUri"]

    filters = list()
    filters.append("enclosureGroupUri='{}'".format(eg_uri))
    filters.append("serverHardwareTypeUri='{}'".format(sht_uri))

    server_profile_templs = g.oneview_client \
        .server_profile_templates.get_all(filter=filters)

    zone_service = ZoneService(g.oneview_client)
    zone_ids = zone_service.get_zone_ids_by_templates(server_profile_templs)

    # Build ResourceBlock object and validates it
    return ServerHardwareResourceBlock(uuid, server_hardware, zone_ids)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from flask import abort
from flask import g

from flask_api import status
from hpOneView.resources.resource import ResourceClient
from oneview_redfish_toolkit.api.computer_system import ComputerSystem


from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection import \
    ResourceBlockCollection

import oneview_redfish_toolkit.api.status_mapping as status_mapping


class Volume(RedfishJsonValidator):
    """Creates a Volume Redfish dict

        Populates self.redfish with Volume data
    """

    SCHEMA_NAME = 'Volume'
    METADATA_INFO = "/redfish/v1/$metadata#Volume.Volume"

    def __init__(self, data):
        """Volume constructor

            Populates self.redfish and validates the result

            Args:
                data: a dict with Redfish's Volume data
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish.update(data)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["@odata.context"] = self.__class__.METADATA_INFO

        self._validate()

    @staticmethod
    def build_volume_for_resource_blocks(resource_block_id, volume):
        """Returns a storage Volume with the contents of data from an Oneview

            Args:
                resource_block_id: storage volume uuid
                volume: storage volume

        """
        attrs = {}
        attrs["@odata.id"] = ResourceBlockCollection.BASE_URI + "/" + \
            resource_block_id + "/Storage/1/Volumes/1"
        attrs["Id"] = "1"
        attrs["Identifiers"] = list()
        attrs["Identifiers"].append({
            "DurableNameFormat": "UUID",
            "DurableName": volume["uri"].split("/")[-1]
        })
        attrs["CapacityBytes"] = int(volume["provisionedCapacity"])
        attrs["Name"] = volume["name"]
        attrs["Status"] = collections.OrderedDict()
        map_struct = status_mapping.STATUS_MAP.get(volume["status"])
        attrs["Status"]["State"] = map_struct["State"]
        attrs["Status"]["Health"] = map_struct["Health"]

        raidlevel = get_raid_level_for_storage_volume(volume)
        if raidlevel:
            attrs["VolumeType"] = status_mapping.RAID_LEVEL.get(raidlevel)
        else:
            attrs["VolumeType"] = "RawDevice"

        return Volume(attrs)

    @staticmethod
    def build_volume_details(uuid, volume_id):
        """Returns a Volume with the contents of data from an Oneview

            Args:
                uuid: Server Profile uuid
                volume_id: Volume id

        """

        server_profile = g.oneview_client.server_profiles.get_by_id(uuid).data
        sas_logical_jbod = get_sas_logical_jbod_by_volumeid(server_profile,
                                                            volume_id)

        if sas_logical_jbod is None:
            abort(status.HTTP_404_NOT_FOUND, "Volume {} not found"
                  .format(volume_id))

        sas_Logical_Interconnect_Uri = \
            sas_logical_jbod["sasLogicalInterconnectUri"]

        drivepaths = []
        for logical_Drive_Bay_Uri in sas_logical_jbod["logicalDriveBayUris"]:
            drivepath = get_drive_path_from_logical_Drive_Bay_Uri(
                logical_Drive_Bay_Uri)
            drivepaths.append(drivepath)

        drive_enclosure_uri = \
            get_drive_enclosure_uri_from_sas_Logical_Interconnect(
                sas_Logical_Interconnect_Uri)

        drive_enclosure_object = get_drive_enclosure_object(
            drive_enclosure_uri)

        drivebayuris = get_drivebayuris_from_drive_enclosure_object(
            drivepaths, drive_enclosure_object)
        device_slot = get_device_slot_from_sas_logical_jbod_by_volumeid(
            server_profile, volume_id)
        raidlevel = get_raidLevel(server_profile, device_slot, volume_id)
        attrs = {}
        attrs["@odata.id"] = ComputerSystem.BASE_URI + "/" + uuid + \
            "/Storage/1/Volumes/" + volume_id
        attrs["Id"] = volume_id
        attrs["Name"] = sas_logical_jbod["name"]
        attrs["Status"] = collections.OrderedDict()
        map_struct = status_mapping.STATUS_MAP.get(sas_logical_jbod["status"])
        attrs["Status"]["State"] = map_struct["State"]
        attrs["Status"]["Health"] = map_struct["Health"]
        if raidlevel is not None:
            attrs["VolumeType"] = status_mapping.RAID_LEVEL.get(raidlevel)
        else:
            attrs["VolumeType"] = "RawDevice"
        attrs["CapacityBytes"] = get_capacity_in_bytes(
            sas_logical_jbod["maxSizeGB"])
        attrs["Identifiers"] = list()
        attrs["Identifiers"].append(
            {"DurableNameFormat": "UUID",
             "DurableName": sas_logical_jbod["uri"].split("/")[-1]})
        attrs["Links"] = collections.OrderedDict()
        attrs["Links"]["Drives"] = list()

        for drivebayuri in drivebayuris:
            attrs["Links"]["Drives"].append(
                {
                    "@odata.id": ComputerSystem.BASE_URI + "/" + uuid +
                    "/Storage/1/Drives/" + drivebayuri.split("/")[-1]
                }
            )

        return Volume(attrs)

    @staticmethod
    def build_external_storage_volume_details(sp_uuid, volume, volume_id):
        """Returns a storage Volume with the contents of data from an Oneview

            Args:
                sp_uuid: server profile uuid
                volume: storage volume
                volume_id: storage_volume_id

        """
        attrs = {}
        attrs["@odata.id"] = ComputerSystem.BASE_URI + "/" + \
            sp_uuid + "/Storage/1/Volumes/" + volume_id
        attrs["Id"] = volume_id
        attrs["Identifiers"] = list()
        attrs["Identifiers"].append({
            "DurableNameFormat": "UUID",
            "DurableName": volume_id
        })
        attrs["CapacityBytes"] = int(volume["provisionedCapacity"])
        attrs["Name"] = volume["name"]
        attrs["Status"] = collections.OrderedDict()
        map_struct = status_mapping.STATUS_MAP.get(volume["status"])
        attrs["Status"]["State"] = map_struct["State"]
        attrs["Status"]["Health"] = map_struct["Health"]

        raidlevel = get_raid_level_for_storage_volume(volume)
        if raidlevel:
            attrs["VolumeType"] = status_mapping.RAID_LEVEL.get(raidlevel)
        else:
            attrs["VolumeType"] = "RawDevice"

        return Volume(attrs)


def get_sas_logical_jbod_by_volumeid(server_profile, volume_id):

    for logical_jbod in server_profile["localStorage"]["sasLogicalJBODs"]:
        if logical_jbod["id"] == int(volume_id) and \
                logical_jbod["sasLogicalJBODUri"]:
            item = g.oneview_client.sas_logical_jbods\
                .get(logical_jbod["sasLogicalJBODUri"])
            return item

    return None


def get_device_slot_from_sas_logical_jbod_by_volumeid(server_profile,
                                                      volume_id):

    for logical_jbod in server_profile["localStorage"]["sasLogicalJBODs"]:
        if logical_jbod["id"] == int(volume_id):
            return logical_jbod["deviceSlot"]

    return None


def get_drive_path_from_logical_Drive_Bay_Uri(logical_Drive_Bay_Uri):
    ov_client = g.oneview_client
    conn = ov_client.connection
    URI = logical_Drive_Bay_Uri
    resource_client = ResourceClient(conn, URI)
    item = resource_client.get(URI)
    return item["drivePaths"][0]


def get_drive_enclosure_uri_from_sas_Logical_Interconnect(
        sas_Logical_Interconnect_Uri):
    item = g.oneview_client.sas_logical_interconnects.get_by_uri(
        sas_Logical_Interconnect_Uri).data
    return item["driveEnclosureUris"][0]


def get_drive_enclosure_object(drive_enclosure_uri):
    item = g.oneview_client.drive_enclosures.get(drive_enclosure_uri)
    return item


def get_capacity_in_bytes(capacity_in_gb):
    size_in_bytes = float(capacity_in_gb) * 1024 * 1024 * 1024
    return int(size_in_bytes)


def get_drivebayuris_from_drive_enclosure_object(drivepaths,
                                                 drive_enclosure_object):
    drivebayuris = []
    for drivebay in drive_enclosure_object["driveBays"]:
        # eliminating drive = NONE condition to avoid TypeError: 'NoneType' object is not subscriptable
        if isinstance(drivebay["drive"], dict):
            for drivepath in drivebay["drive"]["drivePaths"]:
                if drivepath in drivepaths:
                    drivebayuri = drivebay["uri"]
                    drivebayuris.append(drivebayuri)
    return drivebayuris


def get_raidLevel(server_profile, device_slot, volume_id):
    for storagecontroller in server_profile["localStorage"]["controllers"]:
        if(storagecontroller["deviceSlot"] == device_slot):
            for logicaldrive in storagecontroller["logicalDrives"]:
                if logicaldrive["sasLogicalJBODId"] == int(volume_id):
                    raidlevel = logicaldrive["raidLevel"]
                    return raidlevel
    return None


def get_raid_level_for_storage_volume(volume):
    raidLevel = None
    storage_pool = g.oneview_client.storage_pools.get_by_uri(
        volume["storagePoolUri"]).data
    if storage_pool:
        deviceSpecifications = storage_pool["deviceSpecificAttributes"]
        if deviceSpecifications.get("supportedRAIDLevel"):
            raidLevel = deviceSpecifications["supportedRAIDLevel"]
    return raidLevel



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""
Dict of schemas version used by application.
"""

SCHEMAS = {
    "ServiceRoot": "ServiceRoot.v1_3_1.json",
    "ChassisCollection": "ChassisCollection.json",
    "Chassis": "Chassis.v1_7_0.json",
    "CollectionCapabilities": "CollectionCapabilities.v1_0_0.json",
    "CompositionService": "CompositionService.v1_0_1.json",
    "ComputerSystemCollection": "ComputerSystemCollection.json",
    "ComputerSystem": "ComputerSystem.v1_5_0.json",
    "Drive": "Drive.v1_4_0.json",
    "ManagerCollection": "ManagerCollection.json",
    "Manager": "Manager.v1_4_0.json",
    "EthernetInterface": "EthernetInterface.v1_4_0.json",
    "EthernetInterfaceCollection": "EthernetInterfaceCollection.json",
    "EventService": "EventService.v1_1_0.json",
    "EventDestination": "EventDestination.v1_3_0.json",
    "Event": "Event.v1_2_1.json",
    "Thermal": "Thermal.v1_4_0.json",
    "StorageCollection": "StorageCollection.json",
    "Storage": "Storage.v1_4_0.json",
    "Message": "Message.v1_0_5.json",
    "NetworkInterfaceCollection": "NetworkInterfaceCollection.json",
    "NetworkPortCollection": "NetworkPortCollection.json",
    "NetworkDeviceFunctionCollection": "NetworkDeviceFunctionCollection.json",
    "NetworkDeviceFunction": "NetworkDeviceFunction.v1_2_1.json",
    "NetworkInterface": "NetworkInterface.v1_1_0.json",
    "NetworkAdapterCollection": "NetworkAdapterCollection.json",
    "NetworkAdapter": "NetworkAdapter.v1_1_0.json",
    "NetworkPort": "NetworkPort.v1_1_0.json",
    "Processor": "Processor.v1_3_0.json",
    "ProcessorCollection": "ProcessorCollection.json",
    "ResourceBlockCollection": "ResourceBlockCollection.json",
    "ResourceBlock": "ResourceBlock.v1_1_0.json",
    "VLanNetworkInterface": "VLanNetworkInterface.v1_1_1.json",
    "VLanNetworkInterfaceCollection": "VLanNetworkInterfaceCollection.json",
    "ZoneCollection": "ZoneCollection.json",
    "Zone": "Zone.v1_2_0.json",
    "Session": "Session.v1_1_0.json",
    "SessionCollection": "SessionCollection.json",
    "EventDestinationCollection": "EventDestinationCollection.json",
    "SessionService": "SessionService.v1_1_3.json",
    "VolumeCollection": "VolumeCollection.json",
    "Volume": "Volume.v1_0_3.json"
}

REGISTRY = {
    "Base": "Base.1.1.0.json"
}



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class NetworkPortCollection(RedfishJsonValidator):
    """Creates a Network Port Collection Redfish dict

        Populates self.redfish with some hardcoded NetworkPortCollection
        values and Oneview ServerHardware result.
    """

    SCHEMA_NAME = 'NetworkPortCollection'

    def __init__(self, server_hardware, device_id):
        """NetworkPortCollection constructor

            Populates self.redfish and calls _validate()

            Args:
                server_hardware: a server hardware dict from OneView
        """
        super().__init__(self.SCHEMA_NAME)

        physical_ports = self.get_resource_by_id(
            server_hardware["portMap"]["deviceSlots"], "deviceNumber",
            device_id)["physicalPorts"]

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Network Port Collection"
        self.redfish["Members"] = list()

        for port in physical_ports:
            port_link = dict()
            port_link["@odata.id"] = \
                "/redfish/v1/Chassis/{}/NetworkAdapters/{}/NetworkPorts/{}" \
                .format(server_hardware["uuid"],
                        device_id, port["portNumber"])
            self.redfish["Members"].append(port_link)

        self.redfish["Members@odata.count"] = len(physical_ports)
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#NetworkPortCollection" \
            ".NetworkPortCollection"
        self.redfish["@odata.id"] = \
            "/redfish/v1/Chassis/{}/NetworkAdapters/{}/NetworkPorts" \
            .format(server_hardware["uuid"], device_id)

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection import \
    ResourceBlockCollection
from oneview_redfish_toolkit.api import status_mapping


class EthernetInterface(RedfishJsonValidator):
    """Creates a Ethernet Interface Redfish dict

        Populates self.redfish with some hardcoded EthernetInterface
        values and Oneview's Server Profile, Connection and Network
    """

    SCHEMA_NAME = 'EthernetInterface'
    METADATA_INFO = "/redfish/v1/$metadata#EthernetInterface.EthernetInterface"

    def __init__(self, data):
        """EthernetInterfaceCollection constructor

            Populates self.redfish and calls _validate()

            Args:
                server_profile: a server profile dict from OneView
                connection: a connection dict from server profile
                network_attr: a network (or networkSet) dict from
                OneView index resources
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()

        self.redfish.update(data)

        self.redfish["@odata.context"] = self.__class__.METADATA_INFO

        self._validate()

    @staticmethod
    def build(server_profile, connection, network_attr):
        conn_id = str(connection["id"])
        odata_uri = ComputerSystem.BASE_URI + "/" \
            + server_profile["uuid"] + "/EthernetInterfaces/" + conn_id
        attrs = {
            "Id": conn_id,
            "Name": connection["name"],
            "Status": status_mapping.STATUS_MAP[connection["status"]],
            "MACAddress": connection["mac"],
            "SpeedMbps": int(connection["allocatedMbps"])
        }
        if connection["macType"] == "Physical":
            attrs["PermanentMACAddress"] = connection["mac"]

        EthernetInterface._fill_vlan_info(attrs, network_attr, odata_uri)

        attrs["@odata.id"] = odata_uri

        return EthernetInterface(attrs)

    @staticmethod
    def build_resource_block(server_profile_template, connection, network):
        conn_id = str(connection["id"])
        server_profile_template_id = \
            server_profile_template["uri"].split("/")[-1]
        odata_id = ResourceBlockCollection.BASE_URI + "/" \
            + server_profile_template_id \
            + "/EthernetInterfaces/" \
            + conn_id

        attrs = {
            "Id": conn_id,
            "Name": network["name"],
            "SpeedMbps": int(connection["requestedMbps"])
        }

        EthernetInterface._fill_vlan_info(attrs, network, odata_id)

        attrs["@odata.id"] = odata_id

        return EthernetInterface(attrs)

    @staticmethod
    def _fill_vlan_info(attrs_dict, network_attr, base_odata_uri):
        if network_attr["category"] == "network-sets":
            attrs_dict["VLANs"] = {"@odata.id": base_odata_uri + "/VLANs"}
        else:
            attrs_dict["VLAN"] = dict()
            attrs_dict["VLAN"]["VLANEnable"] = True
            attrs_dict["VLAN"]["VLANId"] = \
                int(network_attr["attributes"]["vlan_id"])



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import Blueprint
from flask import g

from oneview_redfish_toolkit.api.zone import Zone
from oneview_redfish_toolkit.api.zone_collection import ZoneCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit.config import \
    COUNTER_LOGGER_NAME
from oneview_redfish_toolkit.services import logging_service
from oneview_redfish_toolkit.services.zone_service import ZoneService
from oneview_redfish_toolkit.single_oneview_context import single_oneview

zone = Blueprint("zone", __name__)


@zone.route(ZoneCollection.BASE_URI + "/<zone_uuid>", methods=["GET"])
@single_oneview
def get_zone(zone_uuid):
    """Get the Redfish Zone.

        Return Resource Zone redfish JSON.

        Logs exception of any error and return Internal
        Server Error or Not Found.

        Returns:
            JSON: Redfish json with Resource Zone.
    """
    template_id, enclosure_id = ZoneService.\
        split_zone_id_to_spt_uuid_and_enclosure_id(zone_uuid)

    profile_template = g.oneview_client.server_profile_templates.get_by_id(
        template_id).data
    sh_type_uri = profile_template['serverHardwareTypeUri']
    enclosure_name = None

    if enclosure_id:
        enclosure = g.oneview_client.enclosures.get_by_id(enclosure_id).data
        enclosure_name = enclosure["name"]
        drives = _get_drives(enclosure)
        sh_filter = [
            "locationUri='{}'".format(enclosure['uri']),
            "serverHardwareTypeUri='{}'".format(sh_type_uri)
        ]
    else:
        drives = []
        enclosure_group_uri = profile_template["enclosureGroupUri"]
        sh_filter = [
            "serverGroupUri='{}'".format(enclosure_group_uri),
            "serverHardwareTypeUri='{}'".format(sh_type_uri)
        ]

    server_hardware_list = g.oneview_client.server_hardware.get_all(
        filter=sh_filter)

    volume_list = g.oneview_client.volumes.get_all()
    filter_volume_list = [volume for volume in volume_list
                          if volume["isShareable"]]

    # Emptying volume list to suppress external storage changes for
    # current release.
    # In future, remove this line to enable external storage support
    filter_volume_list = []

    zone_data = Zone(zone_uuid, profile_template, server_hardware_list,
                     enclosure_name, drives, filter_volume_list)

    sh_count = len(server_hardware_list)
    blocks_count = len(zone_data.redfish["Links"]["ResourceBlocks"])
    logging_service.debug(
        COUNTER_LOGGER_NAME,
        "Drives retrieved: " + str(len(drives)),
        "ServerHardware retrieved: " + str(sh_count),
        "ResourceBlocks listed on Zone: " + str(blocks_count))

    return ResponseBuilder.success(zone_data)


def _get_drives(enclosure):
    drive_encl_assoc_uri = "/rest/index/associations/resources" \
                           "?parenturi={}&category=drive-enclosures"\
        .format(enclosure["uri"])
    drive_encl_assoc = g.oneview_client.connection.get(drive_encl_assoc_uri)
    get_drives_uri = '/rest/index/resources' \
        '?category=drives&count=10000' \
        '&filter="driveEnclosureUri:{}"'
    drives = []
    for member in drive_encl_assoc["members"]:
        drive_encl_uri = member["childResource"]["uri"]
        drives_index_list = g.oneview_client.connection.get(
            get_drives_uri.format(drive_encl_uri))

        drives = [drive for drive in drives_index_list["members"]
                  if drive["attributes"].get('capacityInGB') and
                  int(drive["attributes"]["capacityInGB"]) > 0]

    return drives



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import Blueprint

from oneview_redfish_toolkit.api.session_service import SessionService
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder

session_service = Blueprint("session_service", __name__)


@session_service.route(SessionService.BASE_URI, methods=["GET"])
def get_session_service():
    """Get the Redfish Session Service.

        Get method to return SessionService JSON when
        /redfish/v1/SessionService is requested.

        Returns:
            JSON: JSON with SessionService.
    """
    return ResponseBuilder.success(SessionService())



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class StorageCollection(RedfishJsonValidator):
    """Creates a Storage Collection Redfish dict

        Populates self.redfish with some hardcoded StorageCollection
        values
    """

    SCHEMA_NAME = 'StorageCollection'

    def __init__(self, uuid):
        """StorageCollection constructor

            Populates self.redfish with hardcoded StorageCollection
            values

            Args:
                UUID of server profile
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Storage Collection"
        self.redfish["Members@odata.count"] = 1
        self.redfish["Members"] = list()
        self.redfish["Members"].append(collections.OrderedDict())
        self.redfish["Members"][0]["@odata.id"] = \
            "/redfish/v1/Systems/" + uuid + "/Storage/1"
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#StorageCollection" \
            ".StorageCollection"
        self.redfish["@odata.id"] = "/redfish/v1/Systems/" + uuid + "/Storage"
        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2017) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from flask import Blueprint
from flask import g


from oneview_redfish_toolkit.api.network_device_function_collection import\
    NetworkDeviceFunctionCollection
from oneview_redfish_toolkit.blueprints.util.response_builder \
    import ResponseBuilder

network_device_function_collection = Blueprint(
    "network_device_function_collection", __name__)


@network_device_function_collection.route(
    "/redfish/v1/Chassis/<uuid>/NetworkAdapters/<device_id>"
    "/NetworkDeviceFunctions/", methods=["GET"])
def get_network_device_function_collection(uuid, device_id):
    """Get the Redfish Network Interfaces Collection.

    Return NetworkDeviceFunctionCollection Redfish JSON.
    """

    server_hardware = g.oneview_client.server_hardware.get_by_id(uuid).data

    ndfc = NetworkDeviceFunctionCollection(device_id, server_hardware)

    return ResponseBuilder.success(ndfc)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection \
    import ResourceBlockCollection


class ProcessorCollection(RedfishJsonValidator):
    """Creates a Processor Collection Redfish dict

        Populates self.redfish with some hardcoded ProcessorCollection
        values and with the server hardware response from OneView.
    """

    SCHEMA_NAME = 'ProcessorCollection'

    def __init__(self, server_hardware):
        """ProcessorCollection constructor

            Populates self.redfish with a hardcoded ProcessorCollection
            values and with the server hardware response from OneView.

            Args:
                server_hardware: Server hardware dict.
        """

        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Processors Collection"

        self._fill_members(server_hardware)

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ProcessorCollection.ProcessorCollection"
        self.redfish["@odata.id"] = \
            ResourceBlockCollection.BASE_URI + "/" \
            + server_hardware["uuid"] + "/Systems/1/Processors"

        self._validate()

    def _fill_members(self, server_hardware):
        processor_count = server_hardware["processorCount"]

        self.redfish["Members@odata.count"] = processor_count
        self.redfish["Members"] = list()

        for processor_id in range(processor_count):
            processor = dict()
            processor["@odata.id"] = \
                ResourceBlockCollection.BASE_URI + "/" \
                + server_hardware["uuid"] \
                + "/Systems/1/Processors/" + str(processor_id + 1)

            self.redfish["Members"].append(processor)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import collections
import configparser
import glob
import json
import logging
import logging.config
import os

# Modules own libs
from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishException
from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishResourceNotFoundException
from oneview_redfish_toolkit.api import schemas
from oneview_redfish_toolkit import connection
from oneview_redfish_toolkit import util


API_VERSION = 1200

COUNTER_LOGGER_NAME = 'qtty'
PERFORMANCE_LOGGER_NAME = 'perf'
ONEVIEW_SDK_LOGGER_NAME = 'ovData'


# Globals vars:
#   globals()['config']
#   globals()['registry_dict']
#   globals()['stored_schemas']


def get_config():
    return globals()['config']


def get_oneview_config():
    return dict(get_config().items('oneview_config'))


def get_oneview_multiple_ips():
    ips_config = dict(get_config().items('oneview_config'))['ip'].split(",")
    list_ips = [ip.strip() for ip in ips_config]
    return list_ips


def get_credentials():
    return dict(get_config().items('credentials'))


def get_authentication_mode():
    return get_config().get('redfish', 'authentication_mode')


def auth_mode_is_session():
    return get_authentication_mode() == 'session'


def auth_mode_is_conf():
    return get_authentication_mode() == 'conf'


def get_cherrypy_config():
    cherrypy_config = dict(get_config().items('cherrypy_config'))

    for key, val in cherrypy_config.items():
        if val.isdigit() or (val.startswith('-') and val[1:].isdigit()):
            cherrypy_config[key] = int(val)

    return cherrypy_config


def get_registry_dict():
    return globals()['registry_dict']


def get_stored_schemas():
    return globals()['stored_schemas']


def get_api_version():
    return API_VERSION


def get_composition_settings():
    return dict(get_config().items('redfish-composition'))


def configure_logging(log_file_path):
    """Loads logging.conf file

        Loads logging.conf file to create the logger configuration.

        The logger configuration has two handlers, one of stream
        (show logs in the console) and other of file (save a log file)
        where you can choose one of it in [logger_root : handlers].
        In it you can choose the logger level as well.

        Level: Numeric value
        ---------------------
        CRITICAL: 50
        ERROR:    40
        WARNING:  30
        INFO:     20
        DEBUG:    10
        NOTSET:   00
        ---------------------

        How to use: import logging and logging.exception('message')

        Args:
            log_file_path: logging.conf path.

        Exception:
            Exception: if logging.conf file not found.
    """
    if os.path.isfile(log_file_path) is False:
        raise Exception("Config file {} not found".format(log_file_path))
    else:
        logging.config.fileConfig(log_file_path)


def load_config(conf_file):
    """Loads redfish.conf file

        Loads and parsers the system conf file into config global var
        Loads json schemas into schemas_dict global var
        Established a connection with OneView and sets in as ov_conn
        global var

        Args:
            conf_file: string with the conf file name

        Returns:
            None

        Exception:
            HPOneViewException:
                - if fails to connect to oneview
    """

    config = load_conf_file(conf_file)
    globals()['config'] = config

    # Config file read set global vars
    # Setting ov_config
    # ov_config = dict(config.items('oneview_config'))
    # ov_config['credentials'] = dict(config.items('credentials'))
    # ov_config['api_version'] = API_VERSION
    # globals()['ov_config'] = ov_config

    util.load_event_service_info()

    # Load schemas | Store schemas
    try:
        for ip_oneview in get_oneview_multiple_ips():
            connection.check_oneview_availability(ip_oneview)

        registry_dict = load_registry(
            get_registry_path(),
            schemas.REGISTRY)
        globals()['registry_dict'] = registry_dict

        load_schemas(get_schemas_path())
    except Exception as e:
        raise OneViewRedfishException(
            'Failed to connect to OneView: {}'.format(e)
        )


def load_conf_file(conf_file):
    """Loads and parses conf file

        Loads and parses the module conf file

        Args:
            conf_file: string with the conf file name

        Returns:
            ConfigParser object with conf_file configs
    """

    if not os.path.isfile(conf_file):
        raise OneViewRedfishResourceNotFoundException(
            "File {} not found.".format(conf_file)
        )

    config = configparser.ConfigParser()
    config.optionxform = str
    try:
        config.read(conf_file)
    except Exception:
        raise

    return config


def load_registry(registry_dir, registries):
    """Loads Registries

        Loads all registries using registry_dir directory

        Args:
            registry_dir: string with the directory to load registries from
            registries: dict with registry name as key and registry file_name
                as value. The key will also be the key in the returning dict.

        Returns:
            OrderedDict: A dict containing 'RegistryName': registry_obj

        Exceptions:
            OneviewRedfishResourceNotAccessible:
                - if registry_dir is can't be accessed
    """

    if os.path.isdir(registry_dir) is False:
        raise OneViewRedfishResourceNotFoundException(
            "Directory {} not found.".format(registry_dir)
        )

    if os.access(registry_dir, os.R_OK) is False:
        raise OneViewRedfishResourceNotFoundException(
            "Directory {} not found.".format(registry_dir)
        )

    registries_dict = collections.OrderedDict()
    for key in registries:
        try:
            with open(registry_dir + '/' + registries[key]) as f:
                registries_dict[key] = json.load(f)
        except Exception:
            raise OneViewRedfishResourceNotFoundException(
                "File {} not found.".format(registries[key])
            )

    return registries_dict


def load_schemas(schema_dir):
    """Load all DMTF JSON Schemas

        Load all schemas listed in schemas searching schema_dir directory.

        Args:
            schema_dir: String with the directory to load schemas from.

        Returns:
            Dictionary: A dict containing ('http://redfish.dmtf.org/schemas/
                        v1/<schema_file_name>': schema_obj) pairs
    """
    schema_paths = glob.glob(schema_dir + '/*.json')

    if not schema_paths:
        raise OneViewRedfishResourceNotFoundException(
            "JSON Schemas file not found."
        )

    stored_schemas = dict()

    for path in schema_paths:
        with open(path) as schema_file:
            json_schema = json.load(schema_file)

        if os.name == 'nt':
            file_name = path.split('\\')[-1]
        else:
            file_name = path.split('/')[-1]
        stored_schemas["http://redfish.dmtf.org/schemas/v1/" + file_name] = \
            json_schema

    globals()['stored_schemas'] = stored_schemas


def get_registry_path():
    source = util.get_app_path()
    return os.path.join(source, "registry")


def get_schemas_path():
    source = util.get_app_path()
    return os.path.join(source, "schemas")



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class NetworkDeviceFunctionCollection(RedfishJsonValidator):
    """Creates a Network Device Function Collection Redfish dict

        Populates self.redfish with a NetworkDeviceFunctionCollection
    """

    SCHEMA_NAME = 'NetworkDeviceFunctionCollection'

    def __init__(self, device_id, server_hardware):
        """NetworkDeviceFunctionCollection constructor

            Populates self.redfish and calls _validate()

            Args:
                server_hardware: a server hardware dict from OneView
        """
        super().__init__(self.SCHEMA_NAME)
        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Network Device Function Collection"
        members_count = 0
        self.redfish["Members"] = list()

        physical_ports = self.get_resource_by_id(
            server_hardware["portMap"]["deviceSlots"], "deviceNumber",
            device_id)["physicalPorts"]

        for port in physical_ports:
            physical_port = str(port["portNumber"])
            for virtual_port in port["virtualPorts"]:
                virtual_port_id = "_".join((
                    physical_port,
                    str(virtual_port["portNumber"]),
                    virtual_port["portFunction"]))
                members_count += 1
                device_link = dict()
                device_link["@odata.id"] = \
                    "/redfish/v1/Chassis/" + server_hardware["uuid"] + \
                    "/NetworkAdapters/" + device_id + \
                    "/NetworkDeviceFunctions/" + virtual_port_id
                self.redfish["Members"].append(device_link)
        self.redfish["Members@odata.count"] = members_count
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#NetworkDeviceFunctionCollection" \
            ".NetworkDeviceFunctionCollection"
        self.redfish["@odata.id"] = "/redfish/v1/Chassis/" + \
            server_hardware["uuid"] + "/NetworkAdapters/" + device_id + \
            "/NetworkDeviceFunctions/"

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
from collections import OrderedDict
import logging
import threading
import time

# 3rd party libs
from flask import g
from hpOneView.exceptions import HPOneViewException

# Modules own libs
from oneview_redfish_toolkit.api.errors import NOT_FOUND_ONEVIEW_ERRORS
from oneview_redfish_toolkit import client_session
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit.config import ONEVIEW_SDK_LOGGER_NAME
from oneview_redfish_toolkit.config import PERFORMANCE_LOGGER_NAME
from oneview_redfish_toolkit import single_oneview_context as single

# Globals vars:
#   globals()['map_resources_ov']


lock = threading.Lock()


def init_map_resources():
    """Initialize cached resources map"""
    globals()['map_resources_ov'] = OrderedDict()


def init_map_appliances():
    """Initialize cached appliances map"""
    globals()['map_appliances_ov'] = OrderedDict()


def get_map_resources():
    """Get cached resources map"""
    return globals()['map_resources_ov']


def get_map_appliances():
    """Get cached appliances map"""
    return globals()['map_appliances_ov']


def set_map_resources_entry(resource_id, ip_oneview):
    """Set new cached resource"""
    with lock:
        get_map_resources()[resource_id] = ip_oneview


def cleanup_map_resources_entry(resource_id):
    with lock:
        if resource_id in get_map_resources().keys():
            del get_map_resources()[resource_id]


def set_map_appliances_entry(ip_oneview, appliance_uuid):
    """Set new cached appliance"""
    get_map_appliances()[ip_oneview] = appliance_uuid


def query_ov_client_by_resource(resource_id, resource, function,
                                *args, **kwargs):
    """Query resource on OneViews.

        Query specific resource ID on multiple OneViews.
        Look resource ID on cached map ResourceID->OneViewIP for query
        on specific cached OneView IP.
        If the resource ID is not cached yet it searches on all OneViews.

        Returns:
            dict: OneView resource
    """
    # Get OneView's IP in the single OneView context
    single_oneview_ip = single.is_single_oneview_context() and \
        single.get_single_oneview_ip()
    # Get OneView's IP for cached resource ID
    cached_oneview_ip = get_ov_ip_by_resource(resource_id)

    # Get OneView's IP in the single OneView context or cached by resource ID
    ip_oneview = single_oneview_ip or cached_oneview_ip

    # If resource is not cached yet search in all OneViews
    if not ip_oneview:
        return search_resource_multiple_ov(resource, function, resource_id,
                                           None, *args, **kwargs)

    # If it's Single Oneview context and no IP is saved on context yet
    if single.is_single_oneview_context() and not single_oneview_ip:
        single.set_single_oneview_ip(ip_oneview)

    ov_client = client_session.get_oneview_client(ip_oneview)

    try:
        resp = execute_query_ov_client(ov_client, resource, function,
                                       *args, **kwargs)
    except HPOneViewException as e:
        if e.oneview_response["errorCode"] not in NOT_FOUND_ONEVIEW_ERRORS:
            raise

        cleanup_map_resources_entry(resource_id)
        ov_ips = config.get_oneview_multiple_ips()
        ov_ips.remove(ip_oneview)  # just search in the other ips

        if not ov_ips:
            raise

        return search_resource_multiple_ov(resource, function,
                                           resource_id,
                                           ov_ips, *args,
                                           **kwargs)

    # If it's on Single OneView Context and the resource is not
    # mapped to an OneView IP, then we update cache in advance for
    # future requests for this resource
    if single_oneview_ip and not cached_oneview_ip:
        set_map_resources_entry(resource_id, single_oneview_ip)

    return resp


def get_ov_ip_by_resource(resource_id):
    """Get cached OneView's IP by resource ID"""
    map_resources = get_map_resources()

    return map_resources.get(resource_id)


def search_resource_multiple_ov(resource, function, resource_id, ov_ips,
                                *args, **kwargs):
    """Search resource on multiple OneViews

        Query resource on all OneViews.
        If it's looking for a specific resource:
            -Once resource is found it will cache the resource ID for the
                OneView's IP that was found;
            -If it is not found return NotFound exception.
        If it's looking for all resources(get_all):
            -Always query on all OneViews and return a list appended the
                results for all OneViews

        Args:
            resource: resource type (server_hardware)
            function: resource function name (get_all)
            resource_id: set only if it should look for a specific resource ID
            ov_ips: List of Oneview IPs to search for the resource.
                If None is passed, it will search in all IPs based on the
                toolkit configuration.
            *args: original arguments for the OneView client query
            **kwargs: original keyword arguments for the OneView client query

        Returns:
            OneView resource(s)

        Exceptions:
            HPOneViewException: When occur an error on any OneViews which is
            not an not found error.
    """
    result = []
    error_not_found = []
    single_oneview_ip = single.is_single_oneview_context() and \
        single.get_single_oneview_ip()

    # If it's on Single Oneview Context and there is already an OneView IP
    # on the context, then uses it. If not search on All OneViews
    if not ov_ips and single_oneview_ip:
        list_ov_ips = [single_oneview_ip]
    else:
        list_ov_ips = ov_ips or config.get_oneview_multiple_ips()

    # Loop in all OneView's IP
    for ov_ip in list_ov_ips:

        ov_client = client_session.get_oneview_client(ov_ip)

        try:
            # Query resource on OneView
            expected_resource = \
                execute_query_ov_client(ov_client, resource, function,
                                        *args, **kwargs)

            if expected_resource:
                # If it's looking for a especific resource and was found
                if resource_id:
                    set_map_resources_entry(resource_id, ov_ip)

                    # If it's SingleOneviewContext and there is no OneView IP
                    # on the context, then set OneView's IP on the context
                    if single.is_single_oneview_context() and \
                            not single_oneview_ip:
                        single.set_single_oneview_ip(ov_ip)

                    return expected_resource
                else:
                    # If it's looking for a resource list (get_all)
                    if isinstance(expected_resource, list):
                        result.extend(expected_resource)
                    else:
                        result.append(expected_resource)
        except HPOneViewException as exception:
            # If get any error that is not a notFoundError
            if exception.oneview_response["errorCode"] not in \
                    NOT_FOUND_ONEVIEW_ERRORS:
                logging.exception("Error while searching on multiple "
                                  "OneViews for Oneview {}: {}".
                                  format(ov_ip, exception))
                raise exception

            error_not_found.append(exception)

    # If it's looking for a specific resource returns a NotFound exception
    if resource_id and error_not_found:
        raise error_not_found.pop()

    return result


def execute_query_ov_client(ov_client, resource, function, *args, **kwargs):
    """Execute query for resource on OneView client received as parameter"""
    ov_resource = getattr(ov_client, resource)
    ov_function = getattr(ov_resource, function)

    if logging.getLogger().isEnabledFor(logging.DEBUG):
        start_time = time.time()
        host = ov_client.connection.get_host()

        try:
            result = ov_function(*args, **kwargs)
            msg = "Request to Oneview '%s' calling '%s.%s' with args %s " \
                  "and kwargs %s. Result: %s"
            logging.getLogger(ONEVIEW_SDK_LOGGER_NAME).debug(msg, host,
                                                             resource,
                                                             function, args,
                                                             kwargs, result)
            return result
        finally:
            elapsed_time = time.time() - start_time

            g.elapsed_time_ov += elapsed_time

            logging.getLogger(PERFORMANCE_LOGGER_NAME).debug(
                "Request to Oneview '%s' calling '%s.%s': %s",
                host, resource, function, elapsed_time)

    return ov_function(*args, **kwargs)


def execute_query_function(resource, function, *args, **kwargs):
    ov_function = getattr(resource, function)
    result = ov_function(*args, **kwargs)
    return result



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import copy

from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishInvalidAttributeValueException
from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishResourceNotFoundException

RESET_ALLOWABLE_VALUES_LIST = ["On", "ForceOff", "GracefulShutdown",
                               "GracefulRestart", "ForceRestart",
                               "PushPowerButton"]
RESET_INVALID_VALUES_LIST = ["ForceOn", "Nmi"]
POWER_STATE_MAP = {
    "On": {
        "powerState": "On",
    },
    "ForceOff": {
        "powerState": "Off",
        "powerControl": "PressAndHold"
    },
    "GracefulShutdown": {
        "powerState": "Off",
        "powerControl": "MomentaryPress"
    },
    "GracefulRestart": {
        "powerState": "On",
        "powerControl": "Reset"
    },
    "ForceRestart": {
        "powerState": "On",
        "powerControl": "ColdBoot"
    },
    "PushPowerButton": {
        "powerControl": "MomentaryPress"
    }
}


class OneViewPowerOption(object):

    @staticmethod
    def get_power_state_by_reset_type(reset_type):
        try:
            return copy.copy(POWER_STATE_MAP[reset_type])
        except KeyError:
            raise OneViewRedfishResourceNotFoundException(
                "There is no mapping for {} on the OneView".format(reset_type)
            )

    @staticmethod
    def get_oneview_power_configuration(server_hardware, reset_type):
        """Maps Redfish's power options to OneView's power option

            Maps the known Redfish power options to OneView Power option.
            If a unknown power option shows up it will raise an Exception.

            Args:
                server_hardware: List containing all Oneview's server
                hardware.
                reset_type: Redfish power option.

            Returns:
                dict: Dict with OneView power configuration.

            Exception:
                OneViewRedfishException: raises an exception if
                reset_type is an unmapped value.
        """
        if reset_type in RESET_INVALID_VALUES_LIST:
            raise OneViewRedfishInvalidAttributeValueException(
                "{} not mapped to OneView".format(reset_type)
            )

        power_state_map = OneViewPowerOption.\
            get_power_state_by_reset_type(reset_type)

        if reset_type == "PushPowerButton":
            if server_hardware["powerState"] == "On":
                power_state_map["powerState"] = "Off"
            else:
                power_state_map["powerState"] = "On"

        return power_state_map



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class NetworkInterface(RedfishJsonValidator):
    """Creates a NetworkInterface Redfish dict

        Populates self.redfish with NetworkInterface data retrieved from
        OneView
    """

    SCHEMA_NAME = 'NetworkInterface'

    def __init__(self, device_id, server_profile, server_hardware):
        """NetworkInterface constructor

            Populates self.redfish with the contents of server hardware dict
            from Oneview

            Args:
                device_id: The Network Interface ID
                server_profile: Oneview's server profile dict
                server_hardware: Oneview's server hardware dict
        """
        super().__init__(self.SCHEMA_NAME)
        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = device_id

        self.redfish["Name"] = self.get_resource_by_id(
            server_hardware["portMap"]["deviceSlots"], "deviceNumber",
            device_id)["deviceName"]

        self.redfish["Links"] = dict()
        self.redfish["Links"]["NetworkAdapter"] = dict()
        self.redfish["Links"]["NetworkAdapter"]["@odata.id"] = \
            "/redfish/v1/Chassis/" + server_hardware["uuid"] + \
            "/NetworkAdapters/" + device_id

        self.redfish["NetworkPorts"] = dict()
        self.redfish["NetworkPorts"]["@odata.id"] = \
            "/redfish/v1/Chassis/" + server_hardware["uuid"] + \
            "/NetworkAdapters/" + device_id + "/NetworkPorts/"

        self.redfish["NetworkDeviceFunctions"] = dict()
        self.redfish["NetworkDeviceFunctions"]["@odata.id"] = \
            "/redfish/v1/Chassis/" + server_hardware["uuid"] + \
            "/NetworkAdapters/" + device_id + "/NetworkDeviceFunctions/"

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#NetworkInterface.NetworkInterface"
        self.redfish["@odata.id"] = "/redfish/v1/Systems/" + \
            server_profile["uuid"] + "/NetworkInterfaces/" + device_id

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from functools import reduce
import operator

from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection import \
    ResourceBlockCollection
import oneview_redfish_toolkit.api.status_mapping as status_mapping

DEVICE_PROTOCOLS_MAP = {
    "SasHdd": "SAS",
    "SasSsd": "SAS",
    "SataHdd": "SATA",
    "SataSsd": "SATA",
    "NVMeHdd": "NVMe",
    "NVMeSsd": "NVMe"
}


class Storage(RedfishJsonValidator):
    """Creates a Storage Redfish dict

        Populates self.redfish with Storage data
    """

    SCHEMA_NAME = 'Storage'
    METADATA_INFO = "/redfish/v1/$metadata#Storage.Storage"

    def __init__(self, data):
        """Storage constructor

            Populates self.redfish and validates the result

            Args:
                data: a dict with Redfish's Storage data
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish.update(data)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["@odata.context"] = self.__class__.METADATA_INFO

        self._validate()

    @staticmethod
    def _map_supported_device_protos(drive_technologies):
        supported_device_protocols = set()

        try:
            for drive_option in drive_technologies:
                supported_device_protocols.add(
                    DEVICE_PROTOCOLS_MAP[drive_option])
        except Exception:
            supported_device_protocols.add('')

        return supported_device_protocols

    @staticmethod
    def build_for_composed_system(server_profile, server_hardware_type,
                                  sas_logical_jbods,
                                  external_storage_volumes):
        """Returns a Storage with the contents of data from an Oneview

            Args:
                server_profile: Server Profile from Oneview
                server_hardware_type: Server Hardware Type from Oneview
                sas_logical_jbods: SAS Logical JBODs info from Oneview
                external_storage_volumes: Storage volumes from OneView
        """
        attrs = {}

        drive_technologies = \
            server_hardware_type['storageCapabilities']['driveTechnologies']

        attrs["@odata.id"] = \
            ComputerSystem.BASE_URI + "/" \
            + server_profile["uuid"] + "/Storage/1"
        attrs["StorageControllers@odata.count"] = 1
        attrs["Id"] = "1"
        attrs["Name"] = "Storage Controller"
        attrs["Status"] = collections.OrderedDict()
        ok_struct = status_mapping.STATUS_MAP.get("OK")
        attrs["Status"]["State"] = ok_struct["State"]
        attrs["Status"]["Health"] = ok_struct["Health"]
        attrs["StorageControllers"] = list()

        # adapter storage capabilities (if any)
        for adapter in server_hardware_type['adapters']:
            if adapter['storageCapabilities']:
                new_capability = collections.OrderedDict()
                new_capability["SupportedDeviceProtocols"] = sorted(
                    Storage._map_supported_device_protos(drive_technologies))
                attrs["StorageControllers"].append(new_capability)

        # internal storage capabilities
        storage_controllers = collections.OrderedDict()
        storage_controllers["Manufacturer"] = "HPE"
        storage_controllers["SupportedDeviceProtocols"] = \
            sorted(Storage._map_supported_device_protos(drive_technologies))
        attrs["StorageControllers"].append(storage_controllers)

        count_drives_by_jbod = \
            [int(item["numPhysicalDrives"]) for item in sas_logical_jbods]

        count_drives = reduce(operator.add, count_drives_by_jbod, 0)

        attrs["Drives@odata.count"] = count_drives
        attrs["Drives"] = list()
        for i in range(count_drives):
            drive_id = str(i + 1)
            attrs["Drives"].append({
                "@odata.id": attrs["@odata.id"] + "/Drives/" + drive_id
            })
        if len(sas_logical_jbods) != 0 or external_storage_volumes:
            attrs["Volumes"] = collections.OrderedDict()
            attrs["Volumes"]["@odata.id"] = attrs["@odata.id"] + "/Volumes"

        return Storage(attrs)

    @staticmethod
    def build_for_resource_block(storage_block):
        """Returns a Storage with the contents of devices from an

            Oneview's Drive or storage volume

            Args:
                storage_block: Oneview's Drive or Volume dict
        """
        attrs = {}

        drive_uuid = storage_block["uri"].split("/")[-1]
        odata_id = "{}/{}/Storage/1" \
            .format(ResourceBlockCollection.BASE_URI, drive_uuid)

        attrs["Id"] = "1"
        attrs["Name"] = storage_block["name"]
        attrs["Status"] = status_mapping.STATUS_MAP.get(
            storage_block["status"])

        if storage_block["category"] == "storage-volumes":
            attrs["Volumes"] = {
                "@data.id": odata_id + "/Volumes/1"
            }
        else:
            attrs["Drives"] = [
                {
                    "@odata.id": odata_id + "/Drives/1"
                }
            ]

        attrs["@odata.id"] = odata_id

        return Storage(attrs)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import request
from flask import Response
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# own libs
from oneview_redfish_toolkit.api import scmb
from oneview_redfish_toolkit.api.session import Session
from oneview_redfish_toolkit.api.session_collection import SessionCollection
from oneview_redfish_toolkit.blueprints import subscription
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit import client_session

session = Blueprint('session', __name__)


@session.route(SessionCollection.BASE_URI, methods=["GET"])
def get_collection():
    result = SessionCollection(client_session.get_session_ids())
    return ResponseBuilder.success(result)


@session.route(SessionCollection.BASE_URI + "/" + "<session_id>",
               methods=["GET"])
def get_session(session_id):
    if session_id not in client_session.get_session_ids():
        abort(status.HTTP_404_NOT_FOUND)

    return ResponseBuilder.success(Session(session_id))


@session.route(SessionCollection.BASE_URI + "/" + "<session_id>",
               methods=["DELETE"])
def delete_session(session_id):
    token = request.headers.get('x-auth-token')
    session_for_delete = client_session.get_session_id_by_token(token)

    if session_id != session_for_delete:
        abort(status.HTTP_404_NOT_FOUND)

    client_session.clear_session_by_token(token)

    return Response(status=status.HTTP_204_NO_CONTENT,
                    mimetype="application/json")


@session.route(SessionCollection.BASE_URI, methods=["POST"])
def post_session():
    """Posts Session

        The response to the POST request to create a session includes:

            - An X-Auth-Token header that contains a session auth token that
            the client can use an subsequent requests.
            - A Location header that contains a link to the newly created
            session resource.
            - The JSON response body that contains a full representation of
            the newly created session object.

        Exception:
            HPOneViewException: Invalid username or password.
            return abort(401)

    """

    try:
        try:
            body = request.get_json()
            username = body["UserName"]
            password = body["Password"]
        except Exception:
            error_message = "Invalid JSON key. The JSON request body " \
                            "must have the keys UserName and Password"
            abort(status.HTTP_400_BAD_REQUEST, error_message)

        token, session_id = client_session.login(username, password)

        sess = Session(session_id)

        if subscription.add_subscription_from_file():
            scmb.init_event_service(token)

        return ResponseBuilder.success_201(sess, {
            "Location": sess.redfish["@odata.id"],
            "X-Auth-Token": token
        })

    except HPOneViewException as e:
        logging.exception('Unauthorized error: {}'.format(e))
        abort(status.HTTP_401_UNAUTHORIZED)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oneview_redfish_toolkit.config import \
    COUNTER_LOGGER_NAME
from oneview_redfish_toolkit.services.computer_system_service import \
    ComputerSystemService
from oneview_redfish_toolkit.services import logging_service


class ZoneService(object):
    """Represents a Service class of Zone"""

    def __init__(self, oneview_client):
        """ZoneService constructor

            Args:
                oneview_client: client of Oneview SDK
        """
        self.ov_client = oneview_client

    @staticmethod
    def build_zone_id(spt_uuid_or_uri, enclosure_id_or_uri):
        """Builds a Zone ID based on IDs of server profile template and enclosure

            Args:
                spt_uuid_or_uri: UUID or URI of Server Profile Template
                enclosure_id_or_uri: ID or URI of Enclosure
        """
        spt_id = spt_uuid_or_uri.split("/")[-1]
        encl_id = enclosure_id_or_uri.split("/")[-1]
        return spt_id + "-" + encl_id

    @staticmethod
    def split_zone_id_to_spt_uuid_and_enclosure_id(zone_id):
        """Separates the Zone ID in two IDs:

            - Server Profile Template ID
            - Enclosure ID

            If Enclosure ID has not inside the Zone ID, the Enclosure ID
            returned will be None

            Result is a Tuple (spt_id, enclosure_id)

            Args:
                zone_id: the Zone ID
        """
        # verify if has enclosure id inside the zone_uuid,
        # the uuid has by default only 5 groups separated by hyphen
        uuid_groups = zone_id.split("-")

        if len(uuid_groups) > 5:
            enclosure_id = uuid_groups[-1]
            template_id = str.join("-", uuid_groups[:5])
        else:
            template_id = zone_id
            enclosure_id = None

        return template_id, enclosure_id

    def _get_enclosures_uris_by_template(self, server_profile_template,
                                         logical_encl_list):
        enclosure_uris = []
        for logical_encl in logical_encl_list:
            if logical_encl['enclosureGroupUri'] == \
                server_profile_template['enclosureGroupUri']:
                enclosure_uris += logical_encl['enclosureUris']

        #  the set keeps the elements without repetition
        enclosure_uris = list(set(enclosure_uris))
        enclosure_uris.sort()

        return enclosure_uris

    def get_zone_ids_by_templates(self, server_profile_templates):
        """Returns the Zone IDs by each Server Profile Template inside the list:

            If the list is empty, the result will be empty

            Args:
                server_profile_templates: the list of Server Profile Template
        """
        zone_ids = []
        enclosure_uris_with_valid_drive_enclosures = \
            self._get_enclosures_uris_with_valid_drive_enclosures()
        logical_encl_list = self.ov_client.logical_enclosures.get_all()
        for template in server_profile_templates:
            template_id = template["uri"].split("/")[-1]
            controller = ComputerSystemService.get_storage_controller(template)
            if controller:
                enclosures_uris_by_spt = self._get_enclosures_uris_by_template(
                    template, logical_encl_list)
                enclosures_uris = set(enclosures_uris_by_spt)\
                    .intersection(enclosure_uris_with_valid_drive_enclosures)

                for encl_uri in sorted(enclosures_uris):
                    zone_id = ZoneService.build_zone_id(template_id, encl_uri)
                    zone_ids.append(zone_id)
            else:
                zone_ids.append(template_id)

        return zone_ids

    def _get_enclosures_uris_with_valid_drive_enclosures(self):
        valid_enclosures_uris = list()
        drive_enclosures_list = self.ov_client.drive_enclosures.get_all()

        for drive_encl in drive_enclosures_list:
            # Check if have valid driver enclosure
            if drive_encl["driveBays"]:
                # Get enclosure uri from driver enclosure
                for location_entry in \
                    drive_encl['driveEnclosureLocation']['locationEntries']:
                    if location_entry['type'] == 'Enclosure':
                        valid_enclosures_uris.append(location_entry['value'])
                        break

        drive_encl_count = len(drive_enclosures_list)
        valid_encl_count = len(valid_enclosures_uris)
        logging_service.debug(COUNTER_LOGGER_NAME,
                              "Drive Enclosures retrieved: " +
                              str(drive_encl_count),
                              "Valid Enclosures: " + str(valid_encl_count))

        return valid_enclosures_uris



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from copy import deepcopy

from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishInvalidConditionException
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection import \
    ResourceBlockCollection
import oneview_redfish_toolkit.api.status_mapping as status_mapping
from oneview_redfish_toolkit.api.util.power_option import \
    RESET_ALLOWABLE_VALUES_LIST
from oneview_redfish_toolkit.services.computer_system_service import \
    ComputerSystemService


class ComputerSystem(RedfishJsonValidator):
    """Creates a Computer System Redfish dict

        Populates self.redfish with some predefined Computer System
        contents and with data retrieved from OneView
    """

    SCHEMA_NAME = 'ComputerSystem'
    BASE_URI = '/redfish/v1/Systems'
    METADATA_INFO = "/redfish/v1/$metadata#ComputerSystem.ComputerSystem"

    def __init__(self, data):
        """ComputerSystem constructor

            Populates self.redfish and validates it

            Args:
                data: a dict with Redfish's Computer System data
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish.update(data)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["@odata.context"] = self.__class__.METADATA_INFO

        self._validate()

    @staticmethod
    def _get_highest_status_for_sp_and_sh(sp_status, sh_status):
        all_status = dict()
        all_status[sp_status] = \
            status_mapping.CRITICALITY_STATUS[sp_status]
        all_status[sh_status] = \
            status_mapping.CRITICALITY_STATUS[sh_status]

        highest_status = max(all_status, key=(lambda key: all_status[key]))

        return highest_status

    @staticmethod
    def _map_boot(boot_list):
        """Maps Oneview's boot options to Redfish's boot option

            Maps the known OneView boot options to Redfish boot option.
            If a unknown boot option shows up it will be mapped to None

            Args:
                boot_list: List with OneView boot options

            Returns:
                list with Redfish boot options
        """

        redfish_oneview_boot_map = dict()
        redfish_oneview_boot_map['PXE'] = 'Pxe'
        redfish_oneview_boot_map['CD'] = 'Cd'
        redfish_oneview_boot_map['HardDisk'] = 'Hdd'
        redfish_oneview_boot_map['FibreChannelHba'] = 'RemoteDrive'
        redfish_oneview_boot_map['Floppy'] = 'Floppy'
        redfish_oneview_boot_map['USB'] = 'Usb'
        redfish_boot_list = list()

        try:
            for boot_option in boot_list:
                redfish_boot_list.append(
                    redfish_oneview_boot_map[boot_option])
        except Exception:
            redfish_boot_list.append('None')

        return redfish_boot_list

    @staticmethod
    def build_composed_system(server_hardware, server_hardware_types,
                              server_profile, drives,
                              server_profile_template_uuid, manager_uuid,
                              volumes_uris):
        """Builds and returns a Composed ComputerSystem

            Args:
                server_hardware: ServerHardware dict from OneView
                server_hardware_types: ServerHardwareTypes dict from OneView
                server_profile: ServerProfile dict from OneView.
                drives: Drives list from OneView
                server_profile_template_uuid: ServerProfileTemplate uuid
                manager_uuid: Oneview's current manager uuid
                volumes_uris: External storage volume URIs
        """
        attrs = {}

        base_resource = server_profile
        attrs["Id"] = base_resource["uuid"]
        attrs["Description"] = base_resource["description"]
        attrs["Name"] = base_resource["name"]
        attrs["SystemType"] = "Composed"
        attrs["Manufacturer"] = "HPE"
        attrs["Model"] = server_hardware["model"]
        attrs["SerialNumber"] = server_hardware["serialNumber"]
        attrs["Status"] = collections.OrderedDict()
        health = ComputerSystem._get_highest_status_for_sp_and_sh(
            status_mapping.HEALTH_STATE.get(base_resource["status"]),
            status_mapping.HEALTH_STATE.get(server_hardware["status"])
        )
        state, _ = status_mapping. \
            get_redfish_server_profile_state(base_resource)
        attrs["Status"]["State"] = state
        attrs["Status"]["Health"] = health
        attrs["PowerState"] = server_hardware["powerState"]
        attrs["Boot"] = collections.OrderedDict()
        attrs["Boot"]["BootSourceOverrideTarget@Redfish.AllowableValues"] = \
            ComputerSystem._map_boot(server_hardware_types['bootCapabilities'])
        attrs["BiosVersion"] = server_hardware["romVersion"]
        attrs["ProcessorSummary"] = collections.OrderedDict()
        attrs["ProcessorSummary"]['Count'] = \
            server_hardware["processorCount"]
        attrs["ProcessorSummary"]["Model"] = \
            server_hardware["processorType"]
        attrs["MemorySummary"] = collections.OrderedDict()
        attrs["MemorySummary"]["TotalSystemMemoryGiB"] = \
            server_hardware["memoryMb"] / 1024
        attrs["Storage"] = collections.OrderedDict()
        attrs["Storage"]["@odata.id"] = \
            ComputerSystem.BASE_URI + "/" + base_resource['uuid'] + "/Storage"
        attrs["EthernetInterfaces"] = collections.OrderedDict()
        attrs["EthernetInterfaces"]["@odata.id"] = \
            ComputerSystem.BASE_URI + "/" + \
            base_resource['uuid'] + \
            "/EthernetInterfaces"
        attrs["NetworkInterfaces"] = collections.OrderedDict()
        attrs["NetworkInterfaces"]["@odata.id"] = \
            ComputerSystem.BASE_URI + "/" + \
            base_resource['uuid'] + \
            "/NetworkInterfaces"
        attrs["Links"] = collections.OrderedDict()
        attrs["Links"]["Chassis"] = list()
        attrs["Links"]["Chassis"].append(collections.OrderedDict())
        attrs["Links"]["Chassis"][0]["@odata.id"] = \
            "/redfish/v1/Chassis/" + server_hardware['uuid']
        attrs["Links"]["ManagedBy"] = list()

        if manager_uuid:
            attrs["Links"]["ManagedBy"].append(
                collections.OrderedDict())
            attrs["Links"]["ManagedBy"][0]["@odata.id"] = \
                "/redfish/v1/Managers/" + manager_uuid

        attrs["Links"]["ResourceBlocks"] = ComputerSystem.\
            _build_resource_block_members(drives,
                                          server_hardware,
                                          server_profile_template_uuid,
                                          volumes_uris)
        attrs["Actions"] = collections.OrderedDict()
        attrs["Actions"]["#ComputerSystem.Reset"] = \
            collections.OrderedDict()
        attrs["Actions"]["#ComputerSystem.Reset"]["target"] = \
            ComputerSystem.BASE_URI + "/" + \
            base_resource["uuid"] + \
            "/Actions/ComputerSystem.Reset"
        attrs["Actions"]["#ComputerSystem.Reset"][
            "ResetType@Redfish.AllowableValues"] = \
            RESET_ALLOWABLE_VALUES_LIST
        attrs["@odata.id"] = ComputerSystem.BASE_URI + "/" \
            + base_resource["uuid"]

        return ComputerSystem(attrs)

    @staticmethod
    def build_physical_system(server_hardware, manager_uuid):
        """Builds and returns a Physical ComputerSystem

            Args:
                server_hardware: server hardware dict from OneView
                manager_uuid: Oneview's current manager
        """

        attrs = {}
        server_hardware = server_hardware

        attrs["Id"] = server_hardware["uuid"]
        attrs["Name"] = server_hardware["name"]
        attrs["SystemType"] = "Physical"
        attrs["Manufacturer"] = "HPE"
        attrs["Model"] = server_hardware["model"]
        attrs["SerialNumber"] = server_hardware["serialNumber"]
        attrs["Status"] = collections.OrderedDict()
        state, health = status_mapping. \
            get_redfish_server_hardware_status_struct(server_hardware)
        attrs["Status"]["State"] = state
        attrs["Status"]["Health"] = health
        attrs["PowerState"] = server_hardware["powerState"]
        attrs["BiosVersion"] = server_hardware["romVersion"]

        attrs["ProcessorSummary"] = collections.OrderedDict()
        attrs["ProcessorSummary"]['Count'] = \
            server_hardware["processorCount"]
        attrs["ProcessorSummary"]["Model"] = \
            server_hardware["processorType"]
        attrs["Processors"] = dict()
        attrs["Processors"]["@odata.id"] = \
            ResourceBlockCollection.BASE_URI + "/" \
            + server_hardware["uuid"] + "/Systems/1/Processors"

        attrs["MemorySummary"] = collections.OrderedDict()
        attrs["MemorySummary"]["TotalSystemMemoryGiB"] = \
            server_hardware["memoryMb"] / 1024

        attrs["Links"] = {
            "Chassis": [{
                "@odata.id": "/redfish/v1/Chassis/{}"
                             .format(server_hardware["uuid"])
            }],
            "ManagedBy": []
        }

        if manager_uuid:
            attrs["Links"]["ManagedBy"].append({
                "@odata.id": "/redfish/v1/Managers/" + manager_uuid
            })

        attrs["@odata.id"] = \
            ResourceBlockCollection.BASE_URI + "/" \
            + server_hardware["uuid"] \
            + "/Systems/1"

        return ComputerSystem(attrs)

    @staticmethod
    def build_server_profile(profile_name,
                             profile_description,
                             server_profile_template,
                             system_block,
                             network_blocks,
                             storage_blocks,
                             external_storage_block):
        server_profile = deepcopy(server_profile_template)

        server_profile["name"] = profile_name
        if profile_description:
            server_profile["description"] = profile_description
        server_profile["type"] = "ServerProfileV11"
        server_profile["category"] = "server-profiles"
        server_profile["serverHardwareUri"] = \
            "/rest/server-hardware/" + system_block["uuid"]
        server_profile["localStorage"]["sasLogicalJBODs"] = \
            ComputerSystem._build_sas_logical_jbods(profile_name,
                                                    server_profile_template,
                                                    storage_blocks)

        if external_storage_block or \
                server_profile_template["sanStorage"]["volumeAttachments"]:
            server_profile["sanStorage"]["hostOSType"] = \
                "VMware (ESXi)"
            server_profile["sanStorage"]["manageSanStorage"] = True
            server_profile["sanStorage"]["volumeAttachments"] = \
                ComputerSystem._build_volume_attachments(
                    server_profile_template,
                    external_storage_block)

        server_profile.pop("uri", None)
        server_profile.pop("serverProfileDescription", None)
        server_profile.pop("created", None)
        server_profile.pop("modified", None)
        server_profile.pop("status", None)
        server_profile.pop("state", None)
        server_profile.pop("scopesUri", None)
        server_profile.pop("eTag", None)
        server_profile.pop("description", None)
        if isinstance(server_profile.get("connectionSettings"), dict):
            server_profile["connectionSettings"].pop('manageConnections', None)

        key = 'complianceControl'
        for item in server_profile.keys():
            if isinstance(server_profile[item], dict):
                if (key in server_profile[item].keys()):
                    server_profile[item].pop(key)

        return server_profile

    @staticmethod
    def _build_sas_logical_jbods(profile_name, server_profile_template,
                                 storage_blocks):
        sas_logical_jbods = []

        controller = ComputerSystemService.get_storage_controller(
            server_profile_template)

        if storage_blocks and not controller:
            raise OneViewRedfishInvalidConditionException(
                "The Server Profile Template should have a valid storage "
                "controller to use the Storage Resource Blocks passed"
            )

        for index, storage_block in enumerate(storage_blocks):
            storage_id = index + 1
            attributes = storage_block["attributes"]

            storage = {
                "id": storage_id,
                "name": profile_name + " - Storage " + str(storage_id),
                "deviceSlot": controller["deviceSlot"],
                "numPhysicalDrives": 1,
                "driveMinSizeGB": attributes["capacityInGB"],
                "driveMaxSizeGB": attributes["capacityInGB"],
                "driveTechnology": attributes["interfaceType"].capitalize() +
                attributes["mediaType"].capitalize()
            }

            sas_logical_jbods.append(storage)

        return sas_logical_jbods

    @staticmethod
    def _build_volume_attachments(spt, external_storage_block):
        volume_attachments = []

        # Add storage volume from SPT to volume attachments object
        volume_uris = []
        for volume in spt["sanStorage"]["volumeAttachments"]:
            volume["associatedTemplateAttachmentId"] = None
            volume_attachments.append(volume)
            if volume.get("volumeUri"):
                volume_uris.append(volume["volumeUri"])

        # Create volume attachments object for external storage volume
        if external_storage_block:
            storage_paths = []
            for connection in spt["connectionSettings"]["connections"]:
                if connection["functionType"] == "FibreChannel":
                    storage_path = {
                        "targetSelector": "Auto",
                        "targets": [],
                        "connectionId": connection["id"],
                        "isEnabled": True
                    }
                    storage_paths.append(storage_path)

            for volume in external_storage_block:
                if volume["uri"] not in volume_uris:
                    volume_obj = {
                        "lunType": "Auto",
                        "volumeUri": volume["uri"],
                        "volumeStorageSystemUri": volume["storageSystemUri"],
                        "storagePaths": storage_paths,
                    }
                    volume_attachments.append(volume_obj)

        return volume_attachments

    @staticmethod
    def _build_resource_block_members(drives,
                                      server_hardware,
                                      server_profile_template_uuid,
                                      volumes_uris):
        resource_block_uuids = \
            ComputerSystem._get_resource_block_uuids(
                drives,
                server_hardware,
                server_profile_template_uuid,
                volumes_uris)

        base_uri = ResourceBlockCollection.BASE_URI + "/{}"
        blocks = []
        for resource_block_uuid in resource_block_uuids:
            blocks.append({"@odata.id": base_uri.format(resource_block_uuid)})

        return blocks

    @staticmethod
    def _get_resource_block_uuids(drives,
                                  server_hardware,
                                  server_profile_template_uuid,
                                  volumes_uris):
        resource_block_uuids = list()
        resource_block_uuids.append(server_hardware["uuid"])

        # fill with network resource block
        if server_profile_template_uuid:
            resource_block_uuids.append(server_profile_template_uuid)

        for drive in drives:
            storage_resource_uuid = drive["uri"].split("/")[-1]
            resource_block_uuids.append(storage_resource_uuid)

        for volume_uri in volumes_uris:
            resource_block_uuids.append(volume_uri.split("/")[-1])

        return resource_block_uuids



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import Blueprint

from oneview_redfish_toolkit.api.subscription_collection \
    import SubscriptionCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit import util


subscription_collection = Blueprint("subscription_collection", __name__)


@subscription_collection.route(
    "/redfish/v1/EventService/EventSubscriptions/", methods=["GET"])
def get_subscription_collection():
    """Get the Redfish Subscription Collection.

        Get method to return SubscriptionCollection JSON when
        /redfish/v1/EventService/EventSubscriptions is requested.
        Returns:
                JSON: JSON with EventSubscriptions.
    """

    # Build Subscription Collection object and validates it
    sc = SubscriptionCollection(util.get_all_subscriptions())

    return ResponseBuilder.success(sc)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class ChassisCollection(RedfishJsonValidator):
    """Creates a Chassis Collection Redfish dict

        Populates self.redfish with some hardcoded ChassisCollection
        values and with the response of Oneview with all server hardware,
        enclosures and racks resource.
    """

    SCHEMA_NAME = 'ChassisCollection'

    def __init__(self, server_hardware, enclosures, racks):
        """ChassisCollection constructor

            Populates self.redfish with a hardcoded ChassisCollection
            values and with the response of Oneview with all server hardware,
            enclosures and racks resource.

            Args:
                server_hardware: A list of dicts of server hardware.
                enclosures: A list of dicts of enclosures.
                racks: A list of dicts of racks.
        """

        super().__init__(self.SCHEMA_NAME)

        self.server_hardware = server_hardware
        self.enclosures = enclosures
        self.racks = racks

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "MultiBlade Enclosure Chassis Collection"
        self.redfish["Members@odata.count"] = self. \
            _get_redfish_members_length()
        self.redfish["Members"] = list()
        self._set_redfish_members()
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ChassisCollection" \
            ".ChassisCollection"
        self.redfish["@odata.id"] = "/redfish/v1/Chassis"

        self._validate()

    def _set_redfish_members(self):
        """Mounts the list of Redfish members

            Populates self.redfish["Members"] with the links to Redfish
            EnclosureChassis, BladeChassis and Rack.
        """

        self._set_resource_links(self.enclosures)
        self._set_resource_links(self.racks)
        self._set_resource_links(self.server_hardware)

    def _set_resource_links(self, oneview_resource):
        """Populates self.redfish["Members"] with the links resources"""

        for resource in oneview_resource:
            link_dict = collections.OrderedDict()
            link_dict["@odata.id"] = \
                "/redfish/v1/Chassis/" + resource["uuid"]
            self.redfish["Members"].append(link_dict)

    def _get_redfish_members_length(self):
        """Gets the length of redfish members"""

        return len(self.server_hardware) + len(self.enclosures) \
            + len(self.racks)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishInvalidAttributeValueException
from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishResourceNotFoundException
from oneview_redfish_toolkit.api.redfish_json_validator import \
    RedfishJsonValidator


class NetworkDeviceFunction(RedfishJsonValidator):
    """Creates a NetworkDeviceFunction Redfish dict

        Populates self.redfish with NetworkDeviceFunction data retrieved from
        OneView
    """

    SCHEMA_NAME = 'NetworkDeviceFunction'

    def __init__(self, device_id, device_function_id, server_hardware):
        """NetworkDeviceFunction constructor

            Populates self.redfish with the contents of server hardware dict
            from Oneview

            Args:
                device_id: ID of the NetworkAdapter
                device_function_id: ID of the DeviceFunction. Expected format:
                    portNumber_virtualPortNumber_virtualPortFunction
                server_hardware: Oneview's server hardware dict
        """
        super().__init__(self.SCHEMA_NAME)

        # device_function_id validation
        try:
            port_number, virtual_port_number, virtual_port_function = \
                device_function_id.split("_")

            physical_ports = self.get_resource_by_id(
                server_hardware["portMap"]["deviceSlots"], "deviceNumber",
                device_id)["physicalPorts"]

            port = self.get_resource_by_id(
                physical_ports, "portNumber", port_number)

            virtual_port = self.get_resource_by_id(
                port["virtualPorts"], "portNumber", virtual_port_number)
        except Exception:
            raise OneViewRedfishResourceNotFoundException(
                "NetworkDeviceFunction id {} not found.".format(
                    device_function_id)
            )

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = device_function_id
        self.redfish["Name"] = "Physical port {}, virtual port {}, device "\
            "function {}".format(
                port_number, virtual_port_number, virtual_port_function)

        if port["type"] == "Ethernet":
            self.redfish["Ethernet"] = dict()
            self.redfish["Ethernet"]["MACAddress"] = virtual_port["mac"]
            self.redfish["NetDevFuncType"] = "Ethernet"
        elif port["type"] == "FibreChannel":
            raise OneViewRedfishInvalidAttributeValueException(
                "FibreChannel not implemented"
            )
        else:
            raise OneViewRedfishInvalidAttributeValueException(
                "Type not supported"
            )

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#NetworkDeviceFunction.NetworkDeviceFunction"
        self.redfish["@odata.id"] = "/redfish/v1/Chassis/" + \
            server_hardware["uuid"] + \
            "/NetworkAdapters/" + device_id + \
            "/NetworkDeviceFunctions/" + device_function_id

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import abort
from flask import Blueprint
from flask import g
from flask_api import status

from hpOneView.exceptions import HPOneViewException

from oneview_redfish_toolkit.api.drive import Drive
from oneview_redfish_toolkit.api.resource_block import ResourceBlock
from oneview_redfish_toolkit.api.storage import Storage
from oneview_redfish_toolkit.api.volume import Volume
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder

storage_for_resource_block = Blueprint("storage_for_resource_block", __name__)

FROZEN_ID = "1"


@storage_for_resource_block.route(
    ResourceBlock.BASE_URI +
    "/<resource_block_uuid>/Storage/<storage_id>", methods=["GET"])
def get_storage_details(resource_block_uuid, storage_id):
    """Get the Redfish Storage details of a Storage ResourceBlock for a given ID.

        Return Storage redfish JSON for a given ID.
        Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with Storage detail information.
    """

    if str(storage_id) != FROZEN_ID:
        abort(status.HTTP_404_NOT_FOUND,
              "Storage {} not found for ResourceBlock {}"
              .format(storage_id, resource_block_uuid))

    storage_block = []
    try:
        storage_block = g.oneview_client.volumes.get_by_id(
            resource_block_uuid).data
    except HPOneViewException as e:
        if e.oneview_response["errorCode"] == 'RESOURCE_NOT_FOUND':
            storage_block = g.oneview_client.index_resources.get(
                '/rest/drives/' + resource_block_uuid)
    result = Storage.build_for_resource_block(storage_block)

    return ResponseBuilder.success(result)


@storage_for_resource_block.route(
    ResourceBlock.BASE_URI +
    "/<resource_block_uuid>/Storage/<storage_id>/Drives/<drive_id>",
    methods=["GET"])
def get_storage_drive_details(resource_block_uuid, storage_id, drive_id):
    """Get the Redfish Drive details of a Storage of a Storage Resource Block

        Return Drive redfish JSON for a given ID.
        Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with Drive detail information.
    """

    drive = g.oneview_client.index_resources.get(
        '/rest/drives/' + resource_block_uuid)

    if str(storage_id) != FROZEN_ID:
        abort(status.HTTP_404_NOT_FOUND,
              "Storage {} not found for ResourceBlock {}"
              .format(storage_id, resource_block_uuid))

    if str(drive_id) != FROZEN_ID:
        abort(status.HTTP_404_NOT_FOUND,
              "Drive {} not found for Storage {} of ResourceBlock {}"
              .format(drive_id, storage_id, resource_block_uuid))

    drive_encl_uri = drive["attributes"]["driveEnclosureUri"].split('/')[-1]
    drive_enclosure = g.oneview_client.drive_enclosures.get(drive_encl_uri)
    result = Drive.build_for_resource_block(drive, drive_enclosure)

    return ResponseBuilder.success(result)


@storage_for_resource_block.route(
    ResourceBlock.BASE_URI +
    "/<resource_block_uuid>/Storage/<storage_id>/Volumes/<volume_id>",
    methods=["GET"])
def get_storage_volume_details(resource_block_uuid, storage_id, volume_id):
    """Get the Redfish Volume details of a Storage of a Storage Resource Block

        Return Volume redfish JSON for a given ID.
        Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with Volume detail information.
    """

    volume = g.oneview_client.volumes.get_by_id(resource_block_uuid).data

    if str(storage_id) != FROZEN_ID:
        abort(status.HTTP_404_NOT_FOUND,
              "Storage {} not found for ResourceBlock {}"
              .format(storage_id, resource_block_uuid))

    if str(volume_id) != FROZEN_ID:
        abort(status.HTTP_404_NOT_FOUND,
              "Volume {} not found for Storage {} of ResourceBlock {}"
              .format(volume_id, storage_id, resource_block_uuid))

    result = Volume.build_volume_for_resource_blocks(
        resource_block_uuid, volume)

    return ResponseBuilder.success(result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class VLanNetworkInterface(RedfishJsonValidator):
    """Creates a VLanNetworkInterface Redfish dict

        Populates self.redfish with ethernet network data retrieved
        from OneView
    """

    SCHEMA_NAME = 'VLanNetworkInterface'

    def __init__(self, ethernet_network, endpoint):
        """VLanNetworkInterface constructor

            Populates self.redfish with some hardcoded VLanNetworkInterface
            values and with Ethernet Network data retrieved from OneView.

            Args:
                network: Ethernet network dict from OneView
                endpoint: endpoint uri from original REST
        """

        super().__init__(self.SCHEMA_NAME)

        ethernet_network_id = \
            ethernet_network["uri"].split("/")[-1]

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = ethernet_network_id
        self.redfish["Name"] = ethernet_network["name"]
        self.redfish["VLANEnable"] = True
        self.redfish["VLANId"] = ethernet_network["vlanId"]
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#VLanNetworkInterface"
        self.redfish["@odata.id"] = endpoint

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import appdirs
import logging
import logging.config
import OpenSSL
import os
import pkg_resources
import socket

# Modules own libs
from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishInvalidAttributeValueException
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit.event_dispatcher import EventDispatcher


# Globals vars:
#   globals()['subscriptions_by_type']
#   globals()['all_subscriptions']
#   globals()['delivery_retry_attempts']
#   globals()['delivery_retry_interval']

globals()['subscriptions_by_type'] = {
    "ResourceUpdated": {},
    "ResourceAdded": {},
    "ResourceRemoved": {},
    "Alert": {}
}

globals()['all_subscriptions'] = {}

API_VERSION = 1200

CFG_DIR_NAME = 'oneview-redfish-toolkit'


def get_subscriptions_by_type():
    return globals()['subscriptions_by_type']


def get_all_subscriptions():
    return globals()['all_subscriptions']


def get_delivery_retry_attempts():
    return globals()['delivery_retry_attempts']


def get_delivery_retry_interval():
    return globals()['delivery_retry_interval']


def load_event_service_info():
    """Loads Event Service information

        Loads DeliveryRetryAttempts and DeliveryRetryIntervalSeconds
        from CONFIG file and store it in a global var.

        Exceptions:
            ValueError: DeliveryRetryAttempts and
            DeliveryRetryIntervalSeconds must be integers greater than zero.
    """
    app_config = config.get_config()
    event_service = dict(app_config.items("event_service"))

    try:
        delivery_retry_attempts = \
            int(event_service["DeliveryRetryAttempts"])
        delivery_retry_interval = \
            int(event_service["DeliveryRetryIntervalSeconds"])

        if delivery_retry_attempts <= 0 or delivery_retry_interval <= 0:
            raise OneViewRedfishInvalidAttributeValueException(
                "DeliveryRetryAttempts and DeliveryRetryIntervalSeconds "
                "must be an integer greater than zero."
            )
    except ValueError:
        raise OneViewRedfishInvalidAttributeValueException(
            "DeliveryRetryAttempts and DeliveryRetryIntervalSeconds "
            "must be valid integers."
        )

    globals()['delivery_retry_attempts'] = delivery_retry_attempts
    globals()['delivery_retry_interval'] = delivery_retry_interval


def generate_certificate(dir_name, file_name, key_length, key_type="rsa"):
    """Create self-signed cert and key files

        Args:
            dir_name: name of the directory to store the files
            file_name: name of the files that will be created. It will append
                .crt to certificate file and .key to key file
            key_length: key length in bits
            key_type: crypto type: RSA or DSA; defaults to RSA
        Returns:
            Nothing
        Exceptions:
            Raise exceptions on error
    """

    app_config = config.get_config()
    private_key = OpenSSL.crypto.PKey()
    if key_type == "rsa":
        private_key.generate_key(OpenSSL.crypto.TYPE_RSA, key_length)
    elif key_type == "dsa":
        private_key.generate_key(OpenSSL.crypto.TYPE_DSA, key_length)
    else:
        message = "Invalid key_type"
        logging.error(message)
        raise OneViewRedfishInvalidAttributeValueException(message)

    if not app_config.has_option("ssl-cert-defaults", "commonName"):
        app_config["ssl-cert-defaults"]["commonName"] = get_ip()

    cert = OpenSSL.crypto.X509()
    cert_subject = cert.get_subject()

    cert_defaults = dict(app_config.items("ssl-cert-defaults"))

    for key, value in cert_defaults.items():
        setattr(cert_subject, key, value)

    cert.set_serial_number(1)
    cert.gmtime_adj_notBefore(0)
    cert.gmtime_adj_notAfter(10 * 365 * 24 * 60 * 60)
    cert.set_issuer(cert.get_subject())
    cert.set_pubkey(private_key)
    cert.sign(private_key, "sha1")

    # Save Files
    with open(os.path.join(dir_name, file_name + ".crt"), "wt") as f:
        f.write(OpenSSL.crypto.dump_certificate(
            OpenSSL.crypto.FILETYPE_PEM, cert).decode("UTF-8"))
    with open(os.path.join(dir_name, file_name + ".key"), "wt") as f:
        f.write(OpenSSL.crypto.dump_privatekey(
            OpenSSL.crypto.FILETYPE_PEM, private_key).decode("UTF-8"))


def get_ip():
    """Tries to detect default route IP Address"""
    app_config = config.get_config()
    ov_ip = app_config["oneview_config"]["ip"]
    if ov_ip:
        host_ip = ov_ip
    else:
        host_ip = "8.8.8.8"

    s = socket.socket(type=socket.SOCK_DGRAM)
    try:
        s.connect((host_ip, 1))
        ip = s.getsockname()[0]
    except Exception as e:
        logging.exception(e)
        ip = "127.0.0.1"
    finally:
        s.close()
    return ip


def dispatch_event(event):
    """Creates an EventDispatcher for each subscriber of the event

        Args:
            event: The Event schema describing the JSON payload
            which will be send to the event subscribers.

        Returns:
            Nothing
    """
    # Event resource contains only ONE EventRecord
    events = event.redfish['Events']
    event_record = events[0]

    subscriptions = \
        globals()['subscriptions_by_type'][event_record['EventType']].values()

    for subscription in subscriptions:
        dispatcher = EventDispatcher(
            event,
            subscription,
            globals()['delivery_retry_attempts'],
            globals()['delivery_retry_interval'])

        dispatcher.start()


def get_app_path():
    try:
        source = \
            pkg_resources.resource_filename("oneview_redfish_toolkit", "")
        return source
    except Exception:
        return ""


def get_user_directory():
    return appdirs.user_config_dir(CFG_DIR_NAME)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
from functools import reduce
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import g
from flask import request
from flask import Response
from flask_api import status

# own libs
from hpOneView.exceptions import HPOneViewException
from hpOneView.exceptions import HPOneViewTaskError

from hpOneView.resources.task_monitor import TASK_ERROR_STATES
from jsonschema import ValidationError

from oneview_redfish_toolkit.api.capabilities_object import CapabilitiesObject
from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.util.power_option import OneViewPowerOption
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit.services.computer_system_service import \
    ComputerSystemService
from oneview_redfish_toolkit.services.manager_service import \
    get_manager_uuid
from oneview_redfish_toolkit.single_oneview_context import single_oneview

computer_system = Blueprint("computer_system", __name__)


@computer_system.route("/redfish/v1/Systems/<uuid>", methods=["GET"])
@single_oneview
def get_computer_system(uuid):
    """Get the Redfish Computer System for a given UUID.

        Return ComputerSystem redfish JSON for a given
        server profile or server profile template UUID.

        Returns:
            JSON: Redfish json with ComputerSystem
            When Server hardware or Server profilte templates
            is not found calls abort(404)
    """

    resource = _get_oneview_resource(uuid)
    category = resource["category"]

    if category == 'server-profile-templates':
        computer_system_resource = CapabilitiesObject(resource)
    elif category == 'server-profiles':
        server_hardware = g.oneview_client.server_hardware\
            .get_by_uri(resource["serverHardwareUri"]).data
        server_hardware_type = g.oneview_client.server_hardware_types\
            .get_by_uri(resource['serverHardwareTypeUri']).data

        computer_system_service = ComputerSystemService(g.oneview_client)
        drives = _get_drives_from_sp(resource)
        spt_uuid = computer_system_service.\
            get_server_profile_template_from_sp(resource["uri"])

        # Get external storage volumes from server profile
        volumes_uris = [volume["volumeUri"] for volume in resource[
            "sanStorage"]["volumeAttachments"]]

        # Emptying volume list to suppress external storage changes for
        # current release.
        # In future, remove this line to enable external storage support
        volumes_uris = []

        manager_uuid = get_manager_uuid(resource['serverHardwareTypeUri'])

        # Build Computer System object and validates it
        computer_system_resource = ComputerSystem.build_composed_system(
            server_hardware,
            server_hardware_type,
            resource,
            drives,
            spt_uuid,
            manager_uuid,
            volumes_uris)
    else:
        abort(status.HTTP_404_NOT_FOUND,
              'Computer System UUID {} not found'.format(uuid))

    return ResponseBuilder.success(
        computer_system_resource,
        {"ETag": "W/" + resource["eTag"]})


@computer_system.route("/redfish/v1/Systems/<uuid>/"
                       "Actions/ComputerSystem.Reset", methods=["POST"])
def change_power_state(uuid):
    """Change the Oneview power state for a specific Server hardware.

        Return ResetType Computer System redfish JSON for a
        given server profile UUID.
        Logs exception of any error and return abort.

        Returns:
            JSON: Redfish JSON with ComputerSystem ResetType.
    """

    try:
        reset_type = request.get_json()["ResetType"]
    except KeyError:
        invalid_key = list(request.get_json())[0]  # gets invalid key name
        abort(status.HTTP_400_BAD_REQUEST,
              "Invalid JSON key: {}".format(invalid_key))

    # Gets ServerHardware for given UUID
    profile = g.oneview_client.server_profiles.get_by_id(uuid).data
    sh = g.oneview_client.server_hardware.get_by_uri(
        profile["serverHardwareUri"]).data

    oneview_power_configuration = \
        OneViewPowerOption.get_oneview_power_configuration(
            sh, reset_type)

    # Changes the ServerHardware power state
    g.oneview_client.server_hardware.update_power_state(
        oneview_power_configuration, sh["uuid"])

    return Response(
        response='{"ResetType": "%s"}' % reset_type,
        status=status.HTTP_200_OK,
        mimetype='application/json')


@computer_system.route(
    "/redfish/v1/Systems/<uuid>", methods=["DELETE"])
def remove_computer_system(uuid):
    """Removes a specific System

        Args:
            uuid: The System ID.
    """
    try:
        profile = g.oneview_client.server_profiles.get_by_id(uuid).data
    except HPOneViewException as e:
        abort(status.HTTP_404_NOT_FOUND, e.msg)

    sh_uuid = profile['serverHardwareUri']
    if sh_uuid:
        service = ComputerSystemService(g.oneview_client)
        service.power_off_server_hardware(sh_uuid)

    # Deletes server profile for given UUID
    response = None
    try:
        response = g.oneview_client.server_profiles.delete(uuid)
    except HPOneViewTaskError as e:
        abort(status.HTTP_403_FORBIDDEN, e.msg)

    if response is True:
        return Response(status=status.HTTP_204_NO_CONTENT,
                        mimetype="application/json")

    # Check if returned a task
    if type(response) is dict:
        # Check if task is completed
        if response['taskState'] == 'Completed':
            return Response(status=status.HTTP_204_NO_CONTENT,
                            mimetype="application/json")

        # Log task error messages if it has
        if response['taskState'] in TASK_ERROR_STATES and \
            'taskErrors' in response and len(response['taskErrors']) > 0:
            for err in response['taskErrors']:
                if 'message' in err:
                    logging.exception(err['message'])

    abort(status.HTTP_500_INTERNAL_SERVER_ERROR)


@computer_system.route(ComputerSystem.BASE_URI + "/", methods=["POST"])
def create_composed_system():
    if not request.is_json:
        abort(status.HTTP_400_BAD_REQUEST,
              "The request content should be a valid JSON")

    body = request.get_json()

    result_location_uri = None

    try:
        RedfishJsonValidator.validate(body, 'ComputerSystem')
        service = ComputerSystemService(g.oneview_client)

        blocks = body["Links"]["ResourceBlocks"]
        block_ids = [block["@odata.id"].split("/")[-1] for block in blocks]

        # Should contain only one computer system entry
        system_blocks = _get_system_resource_blocks(block_ids)
        if not system_blocks:
            raise ValidationError(
                "Should have a Computer System Resource Block")

        system_block = system_blocks[0]
        service.validate_computer_system_resource_block_to_composition(
            system_block)

        # Check network block id with the Id attribute in the request
        network_blocks = _get_network_resource_blocks(block_ids)
        spt_id = body["Id"]

        if not (network_blocks and spt_id in network_blocks[0]["uri"]):
            raise ValidationError(
                "Should have a valid Network Resource Block")

        # It can contain zero or more Storage Block
        storage_blocks = _get_storage_resource_blocks(block_ids)

        spt = g.oneview_client.server_profile_templates.get_by_id(spt_id).data

        # Check for external storage block.
        external_storage_blocks = _get_external_storage_block(block_ids)
        if external_storage_blocks:
            is_fibre_channel_nw = _get_fibre_channel_network(spt)
            if is_fibre_channel_nw:
                for volume in external_storage_blocks:
                    storage_pool = g.oneview_client.storage_pools.get_by_uri(
                        volume["storagePoolUri"]).data
                    if storage_pool:
                        storage_system_uri = storage_pool["storageSystemUri"]
                        volume["storageSystemUri"] = storage_system_uri
            else:
                raise ValidationError(
                    "Should have a Fibre Channel Network Connection")

        server_profile = ComputerSystem.build_server_profile(
            body["Name"],
            body.get("Description"),
            spt,
            system_block,
            network_blocks,
            storage_blocks,
            external_storage_blocks)

        service.power_off_server_hardware(system_block["uuid"],
                                          on_compose=True)

        task, resource_uri = service.create_composed_system(server_profile)

        if resource_uri:
            result_uuid = resource_uri.split("/")[-1]
            result_location_uri = ComputerSystem.BASE_URI + "/" + result_uuid
            server_profile_label = dict(
                resourceUri=resource_uri, labels=[spt_id.replace("-", " ")])
            g.oneview_client.labels.create(server_profile_label)
        elif task.get("taskErrors"):
            err_msg = reduce(
                lambda result, msg: result + msg["message"] + "\n",
                task["taskErrors"],
                "")
            abort(status.HTTP_403_FORBIDDEN, err_msg)

    except ValidationError as e:
        abort(status.HTTP_400_BAD_REQUEST, e.message)
    except KeyError as e:
        abort(status.HTTP_400_BAD_REQUEST,
              "Trying access an invalid key {}".format(e.args))
    except HPOneViewTaskError as e:
        abort(status.HTTP_403_FORBIDDEN, e.msg)

    if not result_location_uri:
        logging.error("It was not possible get the server profile URI when "
                      "creating a composed system")
        abort(status.HTTP_500_INTERNAL_SERVER_ERROR)

    return Response(status=status.HTTP_201_CREATED,
                    headers={"Location": result_location_uri},
                    mimetype="application/json")


def _get_oneview_resource(uuid):
    """Gets a Server hardware or Server profile templates"""
    cached_category = category_resource.get_category_by_resource_id(uuid)

    if cached_category:
        resource = getattr(g.oneview_client, cached_category.resource)
        function = getattr(resource, cached_category.function)
        result = function(uuid)
        if isinstance(result, dict):
            return result
        else:
            return result.data

    categories = [
        {"func": g.oneview_client.server_profiles.get_by_id, "param": uuid},
        {"func": g.oneview_client.server_profile_templates.get_by_id, "param": uuid}
    ]

    for category in categories:
        try:
            resource = category["func"](category["param"])
            if isinstance(resource, dict):
                return resource
            return resource.data

        except HPOneViewException as e:
            if e.oneview_response["errorCode"] in \
                    ['RESOURCE_NOT_FOUND', 'ProfileNotFoundException']:
                pass
            else:
                raise  # Raise any unexpected errors

    abort(status.HTTP_404_NOT_FOUND,
          "Could not find computer system with id " + uuid)


def _get_system_resource_blocks(ids):
    return _get_resource_block_data(
        g.oneview_client.server_hardware.get_by_id, ids)


def _get_network_resource_blocks(ids):
    return _get_resource_block_data(
        g.oneview_client.server_profile_templates.get_by_id, ids)


def _get_storage_resource_blocks(ids):
    drive_ids = ["/rest/drives/" + i for i in ids]

    return _get_resource_block_data(
        g.oneview_client.index_resources.get, drive_ids)


def _get_external_storage_block(ids):
    return _get_resource_block_data(
        g.oneview_client.volumes.get_by_id, ids)


def _get_fibre_channel_network(spt):
    is_fibre_channel_nw = False
    for connection in spt["connectionSettings"]["connections"]:
        if connection["functionType"] == "FibreChannel":
            is_fibre_channel_nw = True
            break
    return is_fibre_channel_nw


def _get_resource_block_data(func, uuids):
    resources = list()

    for uuid in uuids:
        try:
            resource = func(uuid)
            if isinstance(resource, dict):
                resources.append(resource)
            else:
                resources.append(resource.data)

        except HPOneViewException as e:
            # With our current implementation, we do not getting
            # server_profile, but if we need get it in some moment,
            # we must check the errorCode 'ProfileNotFoundException'
            if e.oneview_response["errorCode"] == 'RESOURCE_NOT_FOUND':
                pass
            else:
                raise  # Raise any unexpected errors

    return resources


def _get_drives_from_sp(server_profile):
    """Gets Drives from Server Profile"""
    jbods_drives = list()
    for sas_logical_jbod in server_profile["localStorage"]["sasLogicalJBODs"]:
        if sas_logical_jbod["sasLogicalJBODUri"]:
            drives_by_sas_logical_uuid = g.oneview_client.sas_logical_jbods. \
                get_drives(sas_logical_jbod["sasLogicalJBODUri"])
            jbods_drives.extend(drives_by_sas_logical_uuid)

    return jbods_drives



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.capabilities_object import \
    CapabilitiesObject
from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection import \
    ResourceBlockCollection
from oneview_redfish_toolkit.api import status_mapping
from oneview_redfish_toolkit.api.zone_collection import ZoneCollection
from oneview_redfish_toolkit.services.computer_system_service import \
    ComputerSystemService


class Zone(RedfishJsonValidator):
    """Creates a Zone Redfish dict

        Populates self.redfish with Zone data retrieved from
        OneView
    """

    SCHEMA_NAME = 'Zone'

    def __init__(self,
                 resource_id,
                 profile_template,
                 server_hardware_list,
                 enclosure_name,
                 drives=[],
                 volume_list=[]):
        """Zone constructor

            Populates self.redfish with the contents of
            server profile template and server hardware list from Oneview

            Args:
                resource_id: The resource id
                profile_template: Oneview's server profile template dict
                server_hardware_list: Oneview's server hardware list
                (servers and empty bays) for assignment to a server profile
                enclosure_name: Enclosure name associated with a Zone.
                If Zone has not enclosure it will be None.
                drives: Oneview's dict drives list
                volume_list: Oneview's storage volume list
        """
        super().__init__(self.SCHEMA_NAME)

        has_valid_controller = \
            ComputerSystemService.get_storage_controller(profile_template)

        if not has_valid_controller:
            drives = []

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = resource_id
        self.redfish["Name"] = self.get_zone_name(
            profile_template["name"], enclosure_name)
        status_from_ov = profile_template["status"]
        self.redfish["Status"] = status_mapping.STATUS_MAP[status_from_ov]

        self.redfish["Links"] = dict()
        self.redfish["Links"]["ResourceBlocks"] = list()

        self.fill_resource_blocks(profile_template, server_hardware_list,
                                  drives, volume_list)

        self.capabilities_key = "@Redfish.CollectionCapabilities"
        self.redfish[self.capabilities_key] = dict()
        self.redfish[self.capabilities_key]["@odata.type"] = \
            self.get_odata_type_by_schema('CollectionCapabilities')
        self.redfish[self.capabilities_key]["Capabilities"] = list()

        self.fill_capabilities_collection(profile_template)

        self.redfish["@odata.context"] = "/redfish/v1/$metadata#Zone.Zone"
        self.redfish["@odata.id"] = ZoneCollection.BASE_URI + "/" +\
            self.redfish["Id"]

        self._validate()

    @staticmethod
    def get_zone_name(spt_name, enclosure_name):
        zone_name = spt_name
        if enclosure_name:
            zone_name = spt_name + " - " + enclosure_name
        return zone_name

    def fill_resource_blocks(self, profile_template, server_hardware_list,
                             drives, volume_list):
        for item in server_hardware_list:
            self.add_resource_block_item_to_links(item, "uri")

        for item in drives:
            self.add_resource_block_item_to_links(item, "uri")

        for item in volume_list:
            self.add_resource_block_item_to_links(item, "uri")

        self.add_resource_block_item_to_links(profile_template, "uri")

    def add_resource_block_item_to_links(self, original_dict, uri_key):
        uuid = original_dict[uri_key].split("/")[-1]
        dict_item = dict()
        dict_item["@odata.id"] = ResourceBlockCollection.BASE_URI + "/" + uuid
        self.redfish["Links"]["ResourceBlocks"].append(dict_item)

    def fill_capabilities_collection(self, profile_template):
        spt_id = profile_template["uri"].split("/")[-1]
        capability = {
            "CapabilitiesObject": {
                "@odata.id":
                    CapabilitiesObject.BASE_URI + "/" + spt_id
            },
            "UseCase": "ComputerSystemComposition",
            "Links": {
                "TargetCollection": {
                    "@odata.id": ComputerSystem.BASE_URI
                }
            }
        }
        self.redfish[self.capabilities_key]["Capabilities"].append(capability)



#!./redfish-venv/bin/python
# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import collections
import json
import jsonschema

from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishInvalidAttributeValueException
from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishResourceNotFoundException
from oneview_redfish_toolkit.api import schemas
from oneview_redfish_toolkit import config


class RedfishJsonValidator(object):
    """Validates a json object against a Redfish schema

        Base class for redfish classes. Have a builtin validate method and a
        serialize method.
        Creates and empty OrderedDict called redfish which must be populated
        with the redfish data for validation.
        The Serialize method always validates the redfish content before
        generating the json string, It returns the redfish json string on
        successful validation and raises an exception on validation failure
    """

    def __init__(self, schema_name):
        """Constructor

            Adds the schema_obj to itself and creates and empty OrderedDict
            named redfish to be populates with the redfish data to create
            the redfish json

            Args:
                schema_name: The redfish schema name to be used
                            to validate the redfish json created
        """

        self.schema_name = schema_name
        self.redfish = collections.OrderedDict()

    def _validate(self):
        """Validates self.redfish against self.schema_obj

            Validates a redfish OrderedDict against the schema object passed
            on the object creation.

            Returns:
                None

            Exception:
                ValidationError: Raises this exception on validation failure.
        """
        self.validate(self.redfish, self.schema_name)

    @staticmethod
    def validate(dict_to_validate, schema_name):
        """Validates a dict against a schema corresponding to the schema_name

            Returns:
                None

            Exception:
                ValidationError: Raises this exception on validation failure.
        """
        stored_schemas = config.get_stored_schemas()
        schema_obj = RedfishJsonValidator.get_schema_obj(schema_name)

        resolver = jsonschema.RefResolver('', schema_obj, store=stored_schemas)
        jsonschema.validate(dict_to_validate, schema_obj, resolver=resolver)

    def serialize(self):
        """Generates a json string from redfish content

            Serialize the contents of self.redfish. Uses the value of
            indent_json from redfish section of ini file to indent or
            not the result.

            Returns:
                string: json string with the contents of self.redfish
        """

        if config.get_config()['redfish']['indent_json']:
            indent = 4
        else:
            indent = None
        return json.dumps(
            self.redfish,
            default=lambda o: o.__dict__,
            sort_keys=False,
            indent=indent)

    def get_resource_by_id(self, resource_list,
                           resource_number_key, resource_id):
        """Gets a specific resource in the resource list

            Validates the resource ID and gets the resource in
            the resource list.

            Args:
                resource_list: List of resources.
                resource_number_key: Field name of the resource
                    number in the JSON.
                resource_id: Resource's ID that will be searched
                    in the resource list.

            Returns:
                Resource in the list.
        """
        try:
            resource_id = int(resource_id)
        except ValueError:
            raise OneViewRedfishInvalidAttributeValueException(
                "Invalid {} ID".format(self.__class__.__name__)
            )

        for resource in resource_list:
            if resource[resource_number_key] == resource_id:
                return resource

        raise OneViewRedfishResourceNotFoundException(
            "Object {} was not found.".format(self.__class__.__name__)
        )

    def get_odata_type(self):
        """Retrieves odata.type from schema file

            Retrieves the odata.type defined on the schema file
            for the class attribute schema name.

            Returns:
                string: odata.type for the class schema name.
        """
        return self.get_odata_type_by_schema(self.schema_name)

    def get_odata_type_by_schema(self, schema_name):
        """Retrieves odata.type from schema file

            Retrieves the odata.type defined on the schema file
            for the schema name received as parameter.

            Returns:
                string: odata.type for the schema name .
        """
        schema_obj = self.get_schema_obj(schema_name)
        return schema_obj['title']

    @staticmethod
    def get_schema_obj(schema_name):
        """Retrieves schemas object for the schema name

            Retrieves the schema file content loaded
            as a dict for the schema name received
            as parameter.

            Returns:
                dict: schema dict for the schema name.
        """
        schema_file = schemas.SCHEMAS[schema_name]
        stored_schemas = config.get_stored_schemas()

        try:
            schema_obj = stored_schemas[
                "http://redfish.dmtf.org/schemas/v1/" + schema_file]
        except KeyError:
            raise OneViewRedfishResourceNotFoundException(
                "{} not found".format(schema_file)
            )

        return schema_obj



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import collections
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class VolumeCollection(RedfishJsonValidator):

    """Populates the volumes of a server profile in redfish format"""

    SCHEMA_NAME = 'VolumeCollection'

    def __init__(self, server_profile):
        """VolumeCollections constructor

            Populates self.redfish and validates the result

            Args:
                server_profile: server profile object
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Storage Volume Collection"
        self.redfish["Description"] = "Storage Volume Collection"
        self.redfish["Members@odata.count"] = len(
            server_profile["localStorage"]["sasLogicalJBODs"]) + \
            len(server_profile["sanStorage"]["volumeAttachments"])

        self.redfish["Members"] = list()

        for i in range(len(server_profile["localStorage"]["sasLogicalJBODs"])):
            link_dict = collections.OrderedDict()
            link_dict["@odata.id"] = "/redfish/v1/Systems/" \
                + server_profile["uuid"] \
                + "/Storage/1/Volumes/" \
                + str(server_profile["localStorage"]
                      ["sasLogicalJBODs"][i]["id"])
            self.redfish["Members"].append(link_dict)

        for volume in server_profile["sanStorage"]["volumeAttachments"]:
            link_dict = collections.OrderedDict()
            volume_id = volume["volumeUri"].split("/")[-1]
            link_dict["@odata.id"] = "/redfish/v1/Systems/" \
                + server_profile["uuid"] \
                + "/Storage/1/Volumes/" \
                + volume_id
            self.redfish["Members"].append(link_dict)

        self.redfish["Oem"] = {}
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#VolumeCollection" \
            ".VolumeCollection"
        self.redfish["@odata.id"] = "/redfish/v1/Systems/" \
            + server_profile["uuid"] + "/Storage/1/Volumes"
        self._validate()
